# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "RecommendationErrorEnum",
    },
)


class RecommendationErrorEnum(proto.Message):
    r"""Container for enum describing possible errors from applying a
    recommendation.

    """

    class RecommendationError(proto.Enum):
        r"""Enum describing possible errors from applying a
        recommendation.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            BUDGET_AMOUNT_TOO_SMALL (2):
                The specified budget amount is too low for
                example, lower than minimum currency unit or
                lower than ad group minimum cost-per-click.
            BUDGET_AMOUNT_TOO_LARGE (3):
                The specified budget amount is too large.
            INVALID_BUDGET_AMOUNT (4):
                The specified budget amount is not a valid
                amount, for example, not a multiple of minimum
                currency unit.
            POLICY_ERROR (5):
                The specified keyword or ad violates ad
                policy.
            INVALID_BID_AMOUNT (6):
                The specified bid amount is not valid, for
                example, too many fractional digits, or negative
                amount.
            ADGROUP_KEYWORD_LIMIT (7):
                The number of keywords in ad group have
                reached the maximum allowed.
            RECOMMENDATION_ALREADY_APPLIED (8):
                The recommendation requested to apply has
                already been applied.
            RECOMMENDATION_INVALIDATED (9):
                The recommendation requested to apply has
                been invalidated.
            TOO_MANY_OPERATIONS (10):
                The number of operations in a single request
                exceeds the maximum allowed.
            NO_OPERATIONS (11):
                There are no operations in the request.
            DIFFERENT_TYPES_NOT_SUPPORTED (12):
                Operations with multiple recommendation types
                are not supported when partial failure mode is
                not enabled.
            DUPLICATE_RESOURCE_NAME (13):
                Request contains multiple operations with the same
                resource_name.
            RECOMMENDATION_ALREADY_DISMISSED (14):
                The recommendation requested to dismiss has
                already been dismissed.
            INVALID_APPLY_REQUEST (15):
                The recommendation apply request was
                malformed and invalid.
            RECOMMENDATION_TYPE_APPLY_NOT_SUPPORTED (17):
                The type of recommendation requested to apply
                is not supported.
            INVALID_MULTIPLIER (18):
                The target multiplier specified is invalid.
            ADVERTISING_CHANNEL_TYPE_GENERATE_NOT_SUPPORTED (19):
                The passed in advertising_channel_type is not supported.
            RECOMMENDATION_TYPE_GENERATE_NOT_SUPPORTED (20):
                The passed in recommendation_type is not supported.
            RECOMMENDATION_TYPES_CANNOT_BE_EMPTY (21):
                One or more recommendation_types need to be passed into the
                generate recommendations request.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_BIDDING_INFO (22):
                Bidding info is required for the CAMPAIGN_BUDGET
                recommendation type.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_BIDDING_STRATEGY_TYPE (23):
                Bidding strategy type is required for the CAMPAIGN_BUDGET
                recommendation type.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_ASSET_GROUP_INFO (24):
                Asset group info is required for the campaign
                budget recommendation type.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_ASSET_GROUP_INFO_WITH_FINAL_URL (25):
                Asset group info with final url is required for the
                CAMPAIGN_BUDGET recommendation type.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_COUNTRY_CODES_FOR_SEARCH_CHANNEL (26):
                Country codes are required for the CAMPAIGN_BUDGET
                recommendation type for SEARCH channel.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_INVALID_COUNTRY_CODE_FOR_SEARCH_CHANNEL (27):
                Country code is invalid for the CAMPAIGN_BUDGET
                recommendation type for SEARCH channel.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_LANGUAGE_CODES_FOR_SEARCH_CHANNEL (28):
                Language codes are required for the CAMPAIGN_BUDGET
                recommendation type for SEARCH channel.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_EITHER_POSITIVE_OR_NEGATIVE_LOCATION_IDS_FOR_SEARCH_CHANNEL (29):
                Either positive or negative location ids are required for
                the CAMPAIGN_BUDGET recommendation type for SEARCH channel.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_AD_GROUP_INFO_FOR_SEARCH_CHANNEL (30):
                Ad group info is required for the CAMPAIGN_BUDGET
                recommendation type for SEARCH channel.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_KEYWORDS_FOR_SEARCH_CHANNEL (31):
                Keywords are required for the CAMPAIGN_BUDGET recommendation
                type for SEARCH channel.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_WITH_CHANNEL_TYPE_SEARCH_AND_BIDDING_STRATEGY_TYPE_TARGET_IMPRESSION_SHARE_REQUIRES_LOCATION (32):
                Location is required for the CAMPAIGN_BUDGET recommendation
                type for bidding strategy type TARGET_IMPRESSION_SHARE.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_WITH_CHANNEL_TYPE_SEARCH_AND_BIDDING_STRATEGY_TYPE_TARGET_IMPRESSION_SHARE_REQUIRES_TARGET_IMPRESSION_SHARE_MICROS (33):
                Target impression share micros are required for the
                CAMPAIGN_BUDGET recommendation type for bidding strategy
                type TARGET_IMPRESSION_SHARE.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_TARGET_IMPRESSION_SHARE_MICROS_BETWEEN_1_AND_1000000 (34):
                Target impression share micros are required to be between 1
                and 1000000 for the CAMPAIGN_BUDGET recommendation type for
                bidding strategy type TARGET_IMPRESSION_SHARE.
            CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_WITH_CHANNEL_TYPE_SEARCH_AND_BIDDING_STRATEGY_TYPE_TARGET_IMPRESSION_SHARE_REQUIRES_TARGET_IMPRESSION_SHARE_INFO (35):
                Target impression share info is required for the
                CAMPAIGN_BUDGET recommendation type for bidding strategy
                type TARGET_IMPRESSION_SHARE.
            MERCHANT_CENTER_ACCOUNT_ID_NOT_SUPPORTED_ADVERTISING_CHANNEL_TYPE (36):
                Merchant Center Account ID is only supported for
                advertising_channel_type PERFORMANCE_MAX.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BUDGET_AMOUNT_TOO_SMALL = 2
        BUDGET_AMOUNT_TOO_LARGE = 3
        INVALID_BUDGET_AMOUNT = 4
        POLICY_ERROR = 5
        INVALID_BID_AMOUNT = 6
        ADGROUP_KEYWORD_LIMIT = 7
        RECOMMENDATION_ALREADY_APPLIED = 8
        RECOMMENDATION_INVALIDATED = 9
        TOO_MANY_OPERATIONS = 10
        NO_OPERATIONS = 11
        DIFFERENT_TYPES_NOT_SUPPORTED = 12
        DUPLICATE_RESOURCE_NAME = 13
        RECOMMENDATION_ALREADY_DISMISSED = 14
        INVALID_APPLY_REQUEST = 15
        RECOMMENDATION_TYPE_APPLY_NOT_SUPPORTED = 17
        INVALID_MULTIPLIER = 18
        ADVERTISING_CHANNEL_TYPE_GENERATE_NOT_SUPPORTED = 19
        RECOMMENDATION_TYPE_GENERATE_NOT_SUPPORTED = 20
        RECOMMENDATION_TYPES_CANNOT_BE_EMPTY = 21
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_BIDDING_INFO = 22
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_BIDDING_STRATEGY_TYPE = 23
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_ASSET_GROUP_INFO = 24
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_ASSET_GROUP_INFO_WITH_FINAL_URL = (
            25
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_COUNTRY_CODES_FOR_SEARCH_CHANNEL = (
            26
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_INVALID_COUNTRY_CODE_FOR_SEARCH_CHANNEL = (
            27
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_LANGUAGE_CODES_FOR_SEARCH_CHANNEL = (
            28
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_EITHER_POSITIVE_OR_NEGATIVE_LOCATION_IDS_FOR_SEARCH_CHANNEL = (
            29
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_AD_GROUP_INFO_FOR_SEARCH_CHANNEL = (
            30
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_REQUIRES_KEYWORDS_FOR_SEARCH_CHANNEL = (
            31
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_WITH_CHANNEL_TYPE_SEARCH_AND_BIDDING_STRATEGY_TYPE_TARGET_IMPRESSION_SHARE_REQUIRES_LOCATION = (
            32
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_WITH_CHANNEL_TYPE_SEARCH_AND_BIDDING_STRATEGY_TYPE_TARGET_IMPRESSION_SHARE_REQUIRES_TARGET_IMPRESSION_SHARE_MICROS = (
            33
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_TARGET_IMPRESSION_SHARE_MICROS_BETWEEN_1_AND_1000000 = (
            34
        )
        CAMPAIGN_BUDGET_RECOMMENDATION_TYPE_WITH_CHANNEL_TYPE_SEARCH_AND_BIDDING_STRATEGY_TYPE_TARGET_IMPRESSION_SHARE_REQUIRES_TARGET_IMPRESSION_SHARE_INFO = (
            35
        )
        MERCHANT_CENTER_ACCOUNT_ID_NOT_SUPPORTED_ADVERTISING_CHANNEL_TYPE = 36


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates operations to add members to a user list (a.k.a. audience).

The example uses an OfflineUserDataJob, and if requested, runs the job. If a
job ID is specified, the example adds operations to that job. Otherwise, it
creates a new job for the operations.

IMPORTANT: Your application should create a single job containing all of the
operations for a user list. This will be far more efficient than creating and
running multiple jobs that each contain a small set of operations.

This feature is only available to accounts that meet the requirements described
at: https://support.google.com/adspolicy/answer/6299717.
"""

import argparse
import hashlib
import sys
import uuid
from typing import List, Dict, Optional, Union, Iterable

from google.protobuf.any_pb2 import Any
from google.rpc import status_pb2

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.services.user_list_service import (
    UserListServiceClient,
)
from google.ads.googleads.v22.services.services.offline_user_data_job_service import (
    OfflineUserDataJobServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchGoogleAdsStreamResponse,
)
from google.ads.googleads.v22.services.types.user_list_service import (
    UserListOperation,
    MutateUserListsResponse,
)
from google.ads.googleads.v22.resources.types.user_list import UserList
from google.ads.googleads.v22.services.types.offline_user_data_job_service import (
    AddOfflineUserDataJobOperationsRequest,
    OfflineUserDataJobOperation,
    AddOfflineUserDataJobOperationsResponse,
    CreateOfflineUserDataJobResponse,
)
from google.ads.googleads.v22.resources.types.offline_user_data_job import (
    OfflineUserDataJob,
)
from google.ads.googleads.v22.common.types.criteria import (
    AddressInfo,
)
from google.ads.googleads.v22.common.types.offline_user_data import (
    UserData,
    UserIdentifier,
)
from google.ads.googleads.v22.errors.types.errors import (
    GoogleAdsFailure,
    GoogleAdsError,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    run_job: bool,
    user_list_id: Optional[str],
    offline_user_data_job_id: Optional[str],
    ad_user_data_consent: Optional[str],
    ad_personalization_consent: Optional[str],
) -> None:
    """Uses Customer Match to create and add users to a new user list.

    Args:
        client: The Google Ads client.
        customer_id: The ID for the customer that owns the user list.
        run_job: if True, runs the OfflineUserDataJob after adding operations.
            Otherwise, only adds operations to the job.
        user_list_id: ID of an existing user list. If None, a new user list is
            created.
        offline_user_data_job_id: ID of an existing OfflineUserDataJob in the
            PENDING state. If None, a new job is created.
        ad_user_data_consent: The consent status for ad user data for all
            members in the job.
        ad_personalization_consent: The personalization consent status for ad
            user data for all members in the job.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    user_list_resource_name: str

    if not offline_user_data_job_id:
        if user_list_id:
            # Uses the specified Customer Match user list.
            user_list_resource_name = googleads_service.user_list_path(
                customer_id, user_list_id
            )
        else:
            # Creates a Customer Match user list.
            user_list_resource_name = create_customer_match_user_list(
                client, customer_id
            )

    add_users_to_customer_match_user_list(
        client,
        customer_id,
        user_list_resource_name,
        run_job,
        offline_user_data_job_id,
        ad_user_data_consent,
        ad_personalization_consent,
    )


# [START add_customer_match_user_list_3]
def create_customer_match_user_list(
    client: GoogleAdsClient, customer_id: str
) -> str:
    """Creates a Customer Match user list.

    Args:
        client: The Google Ads client.
        customer_id: The ID for the customer that owns the user list.

    Returns:
        The string resource name of the newly created user list.
    """
    # Creates the UserListService client.
    user_list_service_client: UserListServiceClient = client.get_service(
        "UserListService"
    )

    # Creates the user list operation.
    user_list_operation: UserListOperation = client.get_type(
        "UserListOperation"
    )

    # Creates the new user list.
    user_list: UserList = user_list_operation.create
    user_list.name = f"Customer Match list #{uuid.uuid4()}"
    user_list.description = (
        "A list of customers that originated from email and physical addresses"
    )
    # Sets the upload key type to indicate the type of identifier that is used
    # to add users to the list. This field is immutable and required for a
    # CREATE operation.
    user_list.crm_based_user_list.upload_key_type = (
        client.enums.CustomerMatchUploadKeyTypeEnum.CONTACT_INFO
    )
    # Membership life span must be between 0 and 540 days inclusive. See:
    # https://developers.google.com/google-ads/api/reference/rpc/latest/UserList#membership_life_span
    # Sets the membership life span to 30 days.
    user_list.membership_life_span = 30

    response: MutateUserListsResponse = (
        user_list_service_client.mutate_user_lists(
            customer_id=customer_id, operations=[user_list_operation]
        )
    )
    user_list_resource_name: str = response.results[0].resource_name
    print(
        f"User list with resource name '{user_list_resource_name}' was created."
    )

    return user_list_resource_name
    # [END add_customer_match_user_list_3]


# [START add_customer_match_user_list]
def add_users_to_customer_match_user_list(
    client: GoogleAdsClient,
    customer_id: str,
    user_list_resource_name: str,
    run_job: bool,
    offline_user_data_job_id: Optional[str],
    ad_user_data_consent: Optional[str],
    ad_personalization_consent: Optional[str],
) -> None:
    """Uses Customer Match to create and add users to a new user list.

    Args:
        client: The Google Ads client.
        customer_id: The ID for the customer that owns the user list.
        user_list_resource_name: The resource name of the user list to which to
            add users.
        run_job: If true, runs the OfflineUserDataJob after adding operations.
            Otherwise, only adds operations to the job.
        offline_user_data_job_id: ID of an existing OfflineUserDataJob in the
            PENDING state. If None, a new job is created.
        ad_user_data_consent: The consent status for ad user data for all
            members in the job.
        ad_personalization_consent: The personalization consent status for ad
            user data for all members in the job.
    """
    # Creates the OfflineUserDataJobService client.
    offline_user_data_job_service_client: OfflineUserDataJobServiceClient = (
        client.get_service("OfflineUserDataJobService")
    )
    offline_user_data_job_resource_name: str

    if offline_user_data_job_id:
        # Reuses the specified offline user data job.
        offline_user_data_job_resource_name = (
            offline_user_data_job_service_client.offline_user_data_job_path(
                customer_id, offline_user_data_job_id
            )
        )
    else:
        # Creates a new offline user data job.
        offline_user_data_job: OfflineUserDataJob = client.get_type(
            "OfflineUserDataJob"
        )
        offline_user_data_job.type_ = (
            client.enums.OfflineUserDataJobTypeEnum.CUSTOMER_MATCH_USER_LIST
        )
        offline_user_data_job.customer_match_user_list_metadata.user_list = (
            user_list_resource_name
        )

        # Specifies whether user consent was obtained for the data you are
        # uploading. For more details, see:
        # https://www.google.com/about/company/user-consent-policy
        if ad_user_data_consent:
            offline_user_data_job.customer_match_user_list_metadata.consent.ad_user_data = client.enums.ConsentStatusEnum[
                ad_user_data_consent
            ]
        if ad_personalization_consent:
            offline_user_data_job.customer_match_user_list_metadata.consent.ad_personalization = client.enums.ConsentStatusEnum[
                ad_personalization_consent
            ]

        # Issues a request to create an offline user data job.
        create_offline_user_data_job_response: (
            CreateOfflineUserDataJobResponse
        ) = offline_user_data_job_service_client.create_offline_user_data_job(
            customer_id=customer_id, job=offline_user_data_job
        )
        offline_user_data_job_resource_name = (
            create_offline_user_data_job_response.resource_name
        )
        print(
            "Created an offline user data job with resource name: "
            f"'{offline_user_data_job_resource_name}'."
        )

    # Issues a request to add the operations to the offline user data job.

    # Best Practice: This example only adds a few operations, so it only sends
    # one AddOfflineUserDataJobOperations request. If your application is adding
    # a large number of operations, split the operations into batches and send
    # multiple AddOfflineUserDataJobOperations requests for the SAME job. See
    # https://developers.google.com/google-ads/api/docs/remarketing/audience-types/customer-match#customer_match_considerations
    # and https://developers.google.com/google-ads/api/docs/best-practices/quotas#user_data
    # for more information on the per-request limits.
    request: AddOfflineUserDataJobOperationsRequest = client.get_type(
        "AddOfflineUserDataJobOperationsRequest"
    )
    request.resource_name = offline_user_data_job_resource_name
    request.operations = build_offline_user_data_job_operations(client)
    request.enable_partial_failure = True

    # Issues a request to add the operations to the offline user data job.
    response: AddOfflineUserDataJobOperationsResponse = (
        offline_user_data_job_service_client.add_offline_user_data_job_operations(
            request=request
        )
    )

    # Prints the status message if any partial failure error is returned.
    # Note: the details of each partial failure error are not printed here.
    # Refer to the error_handling/handle_partial_failure.py example to learn
    # more.
    # Extracts the partial failure from the response status.
    partial_failure: Union[status_pb2.Status, None] = getattr(
        response, "partial_failure_error", None
    )
    if getattr(partial_failure, "code", None) != 0:
        error_details: Iterable[Any, None] = getattr(
            partial_failure, "details", []
        )
        for error_detail in error_details:
            failure_message: GoogleAdsFailure = client.get_type(
                "GoogleAdsFailure"
            )
            # Retrieve the class definition of the GoogleAdsFailure instance
            # in order to use the "deserialize" class method to parse the
            # error_detail string into a protobuf message object.
            failure_object: GoogleAdsFailure = type(
                failure_message
            ).deserialize(error_detail.value)
            errors: Iterable[GoogleAdsError] = failure_object.errors

            for error in errors:
                print(
                    "A partial failure at index "
                    f"{error.location.field_path_elements[0].index} occurred.\n"
                    f"Error message: {error.message}\n"
                    f"Error code: {error.error_code}"
                )

    print("The operations are added to the offline user data job.")

    if not run_job:
        print(
            "Not running offline user data job "
            f"'{offline_user_data_job_resource_name}', as requested."
        )
        return

    # Issues a request to run the offline user data job for executing all
    # added operations.
    offline_user_data_job_service_client.run_offline_user_data_job(
        resource_name=offline_user_data_job_resource_name
    )

    # Retrieves and displays the job status.
    check_job_status(client, customer_id, offline_user_data_job_resource_name)
    # [END add_customer_match_user_list]


# [START add_customer_match_user_list_2]
def build_offline_user_data_job_operations(
    client: GoogleAdsClient,
) -> List[OfflineUserDataJobOperation]:
    """Creates a raw input list of unhashed user information.

    Each element of the list represents a single user and is a dict containing a
    separate entry for the keys "email", "phone", "first_name", "last_name",
    "country_code", and "postal_code". In your application, this data might come
    from a file or a database.

    Args:
        client: The Google Ads client.

    Returns:
        A list containing the operations.
    """
    # The first user data has an email address and a phone number.
    raw_record_1: Dict[str, str] = {
        "email": "dana@example.com",
        # Phone number to be converted to E.164 format, with a leading '+' as
        # required. This includes whitespace that will be removed later.
        "phone": "+1 800 5550101",
    }

    # The second user data has an email address, a mailing address, and a phone
    # number.
    raw_record_2: Dict[str, str] = {
        # Email address that includes a period (.) before the email domain.
        "email": "alex.2@example.com",
        # Address that includes all four required elements: first name, last
        # name, country code, and postal code.
        "first_name": "Alex",
        "last_name": "Quinn",
        "country_code": "US",
        "postal_code": "94045",
        # Phone number to be converted to E.164 format, with a leading '+' as
        # required.
        "phone": "+1 800 5550102",
    }

    # The third user data only has an email address.
    raw_record_3: Dict[str, str] = {"email": "charlie@example.com"}

    # Adds the raw records to a raw input list.
    raw_records: List[Dict[str, str]] = [
        raw_record_1,
        raw_record_2,
        raw_record_3,
    ]

    operations: List[OfflineUserDataJobOperation] = []
    # Iterates over the raw input list and creates a UserData object for each
    # record.
    for record in raw_records:
        # Creates a UserData object that represents a member of the user list.
        user_data: UserData = client.get_type("UserData")

        # Checks if the record has email, phone, or address information, and
        # adds a SEPARATE UserIdentifier object for each one found. For example,
        # a record with an email address and a phone number will result in a
        # UserData with two UserIdentifiers.

        # IMPORTANT: Since the identifier attribute of UserIdentifier
        # (https://developers.google.com/google-ads/api/reference/rpc/latest/UserIdentifier)
        # is a oneof
        # (https://protobuf.dev/programming-guides/proto3/#oneof-features), you
        # must set only ONE of hashed_email, hashed_phone_number, mobile_id,
        # third_party_user_id, or address-info. Setting more than one of these
        # attributes on the same UserIdentifier will clear all the other members
        # of the oneof. For example, the following code is INCORRECT and will
        # result in a UserIdentifier with ONLY a hashed_phone_number:

        # incorrect_user_identifier = client.get_type("UserIdentifier")
        # incorrect_user_identifier.hashed_email = "..."
        # incorrect_user_identifier.hashed_phone_number = "..."

        # The separate 'if' statements below demonstrate the correct approach
        # for creating a UserData object for a member with multiple
        # UserIdentifiers.

        # Checks if the record has an email address, and if so, adds a
        # UserIdentifier for it.
        if "email" in record:
            user_identifier: UserIdentifier = client.get_type("UserIdentifier")
            user_identifier.hashed_email = normalize_and_hash(
                record["email"], True
            )
            # Adds the hashed email identifier to the UserData object's list.
            user_data.user_identifiers.append(user_identifier)

        # Checks if the record has a phone number, and if so, adds a
        # UserIdentifier for it.
        if "phone" in record:
            user_identifier: UserIdentifier = client.get_type("UserIdentifier")
            user_identifier.hashed_phone_number = normalize_and_hash(
                record["phone"], True
            )
            # Adds the hashed phone number identifier to the UserData object's
            # list.
            user_data.user_identifiers.append(user_identifier)

        # Checks if the record has all the required mailing address elements,
        # and if so, adds a UserIdentifier for the mailing address.
        if "first_name" in record:
            required_keys = ("last_name", "country_code", "postal_code")
            # Checks if the record contains all the other required elements of
            # a mailing address.
            if not all(key in record for key in required_keys):
                # Determines which required elements are missing from the
                # record.
                missing_keys = record.keys() - required_keys
                print(
                    "Skipping addition of mailing address information "
                    "because the following required keys are missing: "
                    f"{missing_keys}"
                )
            else:
                user_identifier: UserIdentifier = client.get_type(
                    "UserIdentifier"
                )
                address_info: AddressInfo = user_identifier.address_info
                address_info.hashed_first_name = normalize_and_hash(
                    record["first_name"], False
                )
                address_info.hashed_last_name = normalize_and_hash(
                    record["last_name"], False
                )
                address_info.country_code = record["country_code"]
                address_info.postal_code = record["postal_code"]
                user_data.user_identifiers.append(user_identifier)

        # If the user_identifiers repeated field is not empty, create a new
        # OfflineUserDataJobOperation and add the UserData to it.
        if user_data.user_identifiers:
            operation: OfflineUserDataJobOperation = client.get_type(
                "OfflineUserDataJobOperation"
            )
            operation.create = user_data
            operations.append(operation)
        # [END add_customer_match_user_list_2]

    return operations


# [START add_customer_match_user_list_4]
def check_job_status(
    client: GoogleAdsClient,
    customer_id: str,
    offline_user_data_job_resource_name: str,
) -> None:
    """Retrieves, checks, and prints the status of the offline user data job.

    If the job is completed successfully, information about the user list is
    printed. Otherwise, a GAQL query will be printed, which can be used to
    check the job status at a later date.

    Offline user data jobs may take 6 hours or more to complete, so checking the
    status periodically, instead of waiting, can be more efficient.

    Args:
        client: The Google Ads client.
        customer_id: The ID for the customer that owns the user list.
        offline_user_data_job_resource_name: The resource name of the offline
            user data job to get the status of.
    """
    query: str = f"""
        SELECT
          offline_user_data_job.resource_name,
          offline_user_data_job.id,
          offline_user_data_job.status,
          offline_user_data_job.type,
          offline_user_data_job.failure_reason,
          offline_user_data_job.customer_match_user_list_metadata.user_list
        FROM offline_user_data_job
        WHERE offline_user_data_job.resource_name =
          '{offline_user_data_job_resource_name}'
        LIMIT 1"""

    # Issues a search request using streaming.
    google_ads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    results: SearchGoogleAdsStreamResponse = google_ads_service.search(
        customer_id=customer_id, query=query
    )
    offline_user_data_job_result: OfflineUserDataJob = next(
        iter(results)
    ).offline_user_data_job
    status_name: str = offline_user_data_job_result.status.name
    user_list_resource_name: str = (
        offline_user_data_job_result.customer_match_user_list_metadata.user_list
    )

    print(
        f"Offline user data job ID '{offline_user_data_job_result.id}' with type "
        f"'{offline_user_data_job_result.type_.name}' has status: {status_name}"
    )

    if status_name == "SUCCESS":
        print_customer_match_user_list_info(
            client, customer_id, user_list_resource_name
        )
    elif status_name == "FAILED":
        print(
            f"\tFailure Reason: {offline_user_data_job_result.failure_reason}"
        )
    elif status_name in ("PENDING", "RUNNING"):
        print(
            "To check the status of the job periodically, use the following "
            f"GAQL query with GoogleAdsService.Search: {query}"
        )
    # [END add_customer_match_user_list_4]


def print_customer_match_user_list_info(
    client: GoogleAdsClient, customer_id: str, user_list_resource_name: str
) -> None:
    """Prints information about the Customer Match user list.

    Args:
        client: The Google Ads client.
        customer_id: The ID for the customer that owns the user list.
        user_list_resource_name: The resource name of the user list to which to
            add users.
    """
    # [START add_customer_match_user_list_5]
    googleads_service_client: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    # Creates a query that retrieves the user list.
    query: str = f"""
        SELECT
          user_list.size_for_display,
          user_list.size_for_search
        FROM user_list
        WHERE user_list.resource_name = '{user_list_resource_name}'"""

    # Issues a search request.
    search_results: SearchGoogleAdsStreamResponse = (
        googleads_service_client.search(customer_id=customer_id, query=query)
    )
    # [END add_customer_match_user_list_5]

    # Prints out some information about the user list.
    user_list: UserList = next(iter(search_results)).user_list
    print(
        "The estimated number of users that the user list "
        f"'{user_list.resource_name}' has is "
        f"{user_list.size_for_display} for Display and "
        f"{user_list.size_for_search} for Search."
    )
    print(
        "Reminder: It may take several hours for the user list to be "
        "populated. Estimates of size zero are possible."
    )


def normalize_and_hash(s: str, remove_all_whitespace: bool) -> str:
    """Normalizes and hashes a string with SHA-256.

    Args:
        s: The string to perform this operation on.
        remove_all_whitespace: If true, removes leading, trailing, and
            intermediate spaces from the string before hashing. If false, only
            removes leading and trailing spaces from the string before hashing.

    Returns:
        A normalized (lowercase, remove whitespace) and SHA-256 hashed string.
    """
    # Normalizes by first converting all characters to lowercase, then trimming
    # spaces.
    if remove_all_whitespace:
        # Removes leading, trailing, and intermediate whitespace.
        s = "".join(s.split())
    else:
        # Removes only leading and trailing spaces.
        s = s.strip().lower()

    # Hashes the normalized string using the hashing algorithm.
    return hashlib.sha256(s.encode()).hexdigest()


if __name__ == "__main__":
    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Adds a customer match user list for specified customer."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The ID for the customer that owns the user list.",
    )
    parser.add_argument(
        "-r",
        "--run_job",
        type=bool,
        required=True,
        help=(
            "If true, runs the OfflineUserDataJob after adding operations. "
            "The default value is False."
        ),
    )
    parser.add_argument(
        "-u",
        "--user_list_id",
        type=str,
        required=False,
        help=(
            "The ID of an existing user list. If not specified, this example "
            "will create a new user list."
        ),
    )
    parser.add_argument(
        "-j",
        "--offline_user_data_job_id",
        type=str,
        required=False,
        help=(
            "The ID of an existing OfflineUserDataJob in the PENDING state. If "
            "not specified, this example will create a new job."
        ),
    )
    parser.add_argument(
        "-d",
        "--ad_user_data_consent",
        type=str,
        choices=[
            e.name
            for e in googleads_client.enums.ConsentStatusEnum
            if e.name not in ("UNSPECIFIED", "UNKNOWN")
        ],
        help=(
            "The data consent status for ad user data for all members in "
            "the job."
        ),
    )
    parser.add_argument(
        "-p",
        "--ad_personalization_consent",
        type=str,
        choices=[
            e.name
            for e in googleads_client.enums.ConsentStatusEnum
            if e.name not in ("UNSPECIFIED", "UNKNOWN")
        ],
        help=(
            "The personalization consent status for ad user data for all "
            "members in the job."
        ),
    )

    args: argparse.Namespace = parser.parse_args()

    try:
        main(
            googleads_client,
            args.customer_id,
            args.run_job,
            args.user_list_id,
            args.offline_user_data_job_id,
            args.ad_user_data_consent,
            args.ad_personalization_consent,
        )
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "OfflineUserDataJobMatchRateRangeEnum",
    },
)


class OfflineUserDataJobMatchRateRangeEnum(proto.Message):
    r"""Container for enum describing reasons match rate ranges for a
    customer match list upload.

    """

    class OfflineUserDataJobMatchRateRange(proto.Enum):
        r"""The match rate range of an offline user data job.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Default value for match rate range.
            MATCH_RANGE_LESS_THAN_20 (2):
                Match rate range for offline data upload
                entity is between 0% and 19%.
            MATCH_RANGE_20_TO_30 (3):
                Match rate range for offline data upload
                entity is between 20% and 30%.
            MATCH_RANGE_31_TO_40 (4):
                Match rate range for offline data upload
                entity is between 31% and 40%.
            MATCH_RANGE_41_TO_50 (5):
                Match rate range for offline data upload
                entity is between 41% and 50%.
            MATCH_RANGE_51_TO_60 (6):
                Match rate range for offline data upload
                entity is between 51% and 60%.
            MATCH_RANGE_61_TO_70 (7):
                Match rate range for offline data upload
                entity is between 61% and 70%.
            MATCH_RANGE_71_TO_80 (8):
                Match rate range for offline data upload
                entity is between 71% and 80%.
            MATCH_RANGE_81_TO_90 (9):
                Match rate range for offline data upload
                entity is between 81% and 90%.
            MATCH_RANGE_91_TO_100 (10):
                Match rate range for offline data upload
                entity is more than or equal to 91%.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        MATCH_RANGE_LESS_THAN_20 = 2
        MATCH_RANGE_20_TO_30 = 3
        MATCH_RANGE_31_TO_40 = 4
        MATCH_RANGE_41_TO_50 = 5
        MATCH_RANGE_51_TO_60 = 6
        MATCH_RANGE_61_TO_70 = 7
        MATCH_RANGE_71_TO_80 = 8
        MATCH_RANGE_81_TO_90 = 9
        MATCH_RANGE_91_TO_100 = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.resources.types import custom_interest
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "MutateCustomInterestsRequest",
        "CustomInterestOperation",
        "MutateCustomInterestsResponse",
        "MutateCustomInterestResult",
    },
)


class MutateCustomInterestsRequest(proto.Message):
    r"""Request message for
    [CustomInterestService.MutateCustomInterests][google.ads.googleads.v21.services.CustomInterestService.MutateCustomInterests].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose custom
            interests are being modified.
        operations (MutableSequence[google.ads.googleads.v21.services.types.CustomInterestOperation]):
            Required. The list of operations to perform
            on individual custom interests.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["CustomInterestOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="CustomInterestOperation",
        )
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class CustomInterestOperation(proto.Message):
    r"""A single operation (create, update) on a custom interest.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v21.resources.types.CustomInterest):
            Create operation: No resource name is
            expected for the new custom interest.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v21.resources.types.CustomInterest):
            Update operation: The custom interest is
            expected to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    create: custom_interest.CustomInterest = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=custom_interest.CustomInterest,
    )
    update: custom_interest.CustomInterest = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=custom_interest.CustomInterest,
    )


class MutateCustomInterestsResponse(proto.Message):
    r"""Response message for custom interest mutate.

    Attributes:
        results (MutableSequence[google.ads.googleads.v21.services.types.MutateCustomInterestResult]):
            All results for the mutate.
    """

    results: MutableSequence["MutateCustomInterestResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="MutateCustomInterestResult",
        )
    )


class MutateCustomInterestResult(proto.Message):
    r"""The result for the custom interest mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "MonthOfYearEnum",
    },
)


class MonthOfYearEnum(proto.Message):
    r"""Container for enumeration of months of the year, for example,
    "January".

    """

    class MonthOfYear(proto.Enum):
        r"""Enumerates months of the year, for example, "January".

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            JANUARY (2):
                January.
            FEBRUARY (3):
                February.
            MARCH (4):
                March.
            APRIL (5):
                April.
            MAY (6):
                May.
            JUNE (7):
                June.
            JULY (8):
                July.
            AUGUST (9):
                August.
            SEPTEMBER (10):
                September.
            OCTOBER (11):
                October.
            NOVEMBER (12):
                November.
            DECEMBER (13):
                December.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        JANUARY = 2
        FEBRUARY = 3
        MARCH = 4
        APRIL = 5
        MAY = 6
        JUNE = 7
        JULY = 8
        AUGUST = 9
        SEPTEMBER = 10
        OCTOBER = 11
        NOVEMBER = 12
        DECEMBER = 13


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.enums.types import campaign_draft_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "CampaignDraft",
    },
)


class CampaignDraft(proto.Message):
    r"""A campaign draft.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign draft. Campaign
            draft resource names have the form:

            ``customers/{customer_id}/campaignDrafts/{base_campaign_id}~{draft_id}``
        draft_id (int):
            Output only. The ID of the draft.

            This field is read-only.

            This field is a member of `oneof`_ ``_draft_id``.
        base_campaign (str):
            Immutable. The base campaign to which the
            draft belongs.

            This field is a member of `oneof`_ ``_base_campaign``.
        name (str):
            The name of the campaign draft.

            This field is required and should not be empty
            when creating new campaign drafts.

            It must not contain any null (code point 0x0),
            NL line feed (code point 0xA) or carriage return
            (code point 0xD) characters.

            This field is a member of `oneof`_ ``_name``.
        draft_campaign (str):
            Output only. Resource name of the Campaign
            that results from overlaying the draft changes
            onto the base campaign.

            This field is read-only.

            This field is a member of `oneof`_ ``_draft_campaign``.
        status (google.ads.googleads.v19.enums.types.CampaignDraftStatusEnum.CampaignDraftStatus):
            Output only. The status of the campaign
            draft. This field is read-only.
            When a new campaign draft is added, the status
            defaults to PROPOSED.
        has_experiment_running (bool):
            Output only. Whether there is an experiment
            based on this draft currently serving.

            This field is a member of `oneof`_ ``_has_experiment_running``.
        long_running_operation (str):
            Output only. The resource name of the
            long-running operation that can be used to poll
            for completion of draft promotion. This is only
            set if the draft promotion is in progress or
            finished.

            This field is a member of `oneof`_ ``_long_running_operation``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    draft_id: int = proto.Field(
        proto.INT64,
        number=9,
        optional=True,
    )
    base_campaign: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    draft_campaign: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    status: (
        campaign_draft_status.CampaignDraftStatusEnum.CampaignDraftStatus
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=campaign_draft_status.CampaignDraftStatusEnum.CampaignDraftStatus,
    )
    has_experiment_running: bool = proto.Field(
        proto.BOOL,
        number=13,
        optional=True,
    )
    long_running_operation: str = proto.Field(
        proto.STRING,
        number=14,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "CampaignErrorEnum",
    },
)


class CampaignErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign errors."""

    class CampaignError(proto.Enum):
        r"""Enum describing possible campaign errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CANNOT_TARGET_CONTENT_NETWORK (3):
                Cannot target content network.
            CANNOT_TARGET_SEARCH_NETWORK (4):
                Cannot target search network.
            CANNOT_TARGET_SEARCH_NETWORK_WITHOUT_GOOGLE_SEARCH (5):
                Cannot cover search network without google
                search network.
            CANNOT_TARGET_GOOGLE_SEARCH_FOR_CPM_CAMPAIGN (6):
                Cannot target Google Search network for a CPM
                campaign.
            CAMPAIGN_MUST_TARGET_AT_LEAST_ONE_NETWORK (7):
                Must target at least one network.
            CANNOT_TARGET_PARTNER_SEARCH_NETWORK (8):
                Only some Google partners are allowed to
                target partner search network.
            CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CRITERIA_LEVEL_BIDDING_STRATEGY (9):
                Cannot target content network only as
                campaign has criteria-level bidding strategy.
            CAMPAIGN_DURATION_MUST_CONTAIN_ALL_RUNNABLE_TRIALS (10):
                Cannot modify the start or end date such that
                the campaign duration would not contain the
                durations of all runnable trials.
            CANNOT_MODIFY_FOR_TRIAL_CAMPAIGN (11):
                Cannot modify dates, budget or status of a
                trial campaign.
            DUPLICATE_CAMPAIGN_NAME (12):
                Trying to modify the name of an active or
                paused campaign, where the name is already
                assigned to another active or paused campaign.
            INCOMPATIBLE_CAMPAIGN_FIELD (13):
                Two fields are in conflicting modes.
            INVALID_CAMPAIGN_NAME (14):
                Campaign name cannot be used.
            INVALID_AD_SERVING_OPTIMIZATION_STATUS (15):
                Given status is invalid.
            INVALID_TRACKING_URL (16):
                Error in the campaign level tracking URL.
            CANNOT_SET_BOTH_TRACKING_URL_TEMPLATE_AND_TRACKING_SETTING (17):
                Cannot set both tracking URL template and
                tracking setting. A user has to clear legacy
                tracking setting in order to add tracking URL
                template.
            MAX_IMPRESSIONS_NOT_IN_RANGE (18):
                The maximum number of impressions for
                Frequency Cap should be an integer greater than
                0.
            TIME_UNIT_NOT_SUPPORTED (19):
                Only the Day, Week and Month time units are
                supported.
            INVALID_OPERATION_IF_SERVING_STATUS_HAS_ENDED (20):
                Operation not allowed on a campaign whose
                serving status has ended
            BUDGET_CANNOT_BE_SHARED (21):
                This budget is exclusively linked to a
                Campaign that is using experiments so it cannot
                be shared.
            CAMPAIGN_CANNOT_USE_SHARED_BUDGET (22):
                Campaigns using experiments cannot use a
                shared budget.
            CANNOT_CHANGE_BUDGET_ON_CAMPAIGN_WITH_TRIALS (23):
                A different budget cannot be assigned to a
                campaign when there are running or scheduled
                trials.
            CAMPAIGN_LABEL_DOES_NOT_EXIST (24):
                No link found between the campaign and the
                label.
            CAMPAIGN_LABEL_ALREADY_EXISTS (25):
                The label has already been attached to the
                campaign.
            MISSING_SHOPPING_SETTING (26):
                A ShoppingSetting was not found when creating
                a shopping campaign.
            INVALID_SHOPPING_SALES_COUNTRY (27):
                The country in shopping setting is not an
                allowed country.
            ADVERTISING_CHANNEL_TYPE_NOT_AVAILABLE_FOR_ACCOUNT_TYPE (31):
                The requested channel type is not available
                according to the customer's account setting.
            INVALID_ADVERTISING_CHANNEL_SUB_TYPE (32):
                The AdvertisingChannelSubType is not a valid
                subtype of the primary channel type.
            AT_LEAST_ONE_CONVERSION_MUST_BE_SELECTED (33):
                At least one conversion must be selected.
            CANNOT_SET_AD_ROTATION_MODE (34):
                Setting ad rotation mode for a campaign is
                not allowed. Ad rotation mode at campaign is
                deprecated.
            CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED (35):
                Trying to change start date on a campaign
                that has started.
            CANNOT_SET_DATE_TO_PAST (36):
                Trying to modify a date into the past.
            MISSING_HOTEL_CUSTOMER_LINK (37):
                Hotel center id in the hotel setting does not
                match any customer links.
            INVALID_HOTEL_CUSTOMER_LINK (38):
                Hotel center id in the hotel setting must
                match an active customer link.
            MISSING_HOTEL_SETTING (39):
                Hotel setting was not found when creating a
                hotel ads campaign.
            CANNOT_USE_SHARED_CAMPAIGN_BUDGET_WHILE_PART_OF_CAMPAIGN_GROUP (40):
                A Campaign cannot use shared campaign budgets
                and be part of a campaign group.
            APP_NOT_FOUND (41):
                The app ID was not found.
            SHOPPING_ENABLE_LOCAL_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE (42):
                Campaign.shopping_setting.enable_local is not supported for
                the specified campaign type.
            MERCHANT_NOT_ALLOWED_FOR_COMPARISON_LISTING_ADS (43):
                The merchant does not support the creation of
                campaigns for Shopping Comparison Listing Ads.
            INSUFFICIENT_APP_INSTALLS_COUNT (44):
                The App campaign for engagement cannot be
                created because there aren't enough installs.
            SENSITIVE_CATEGORY_APP (45):
                The App campaign for engagement cannot be
                created because the app is sensitive.
            HEC_AGREEMENT_REQUIRED (46):
                Customers with Housing, Employment, or Credit
                ads must accept updated personalized ads policy
                to continue creating campaigns.
            NOT_COMPATIBLE_WITH_VIEW_THROUGH_CONVERSION_OPTIMIZATION (49):
                The field is not compatible with view through
                conversion optimization.
            INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE (48):
                The field type cannot be excluded because an
                active campaign-asset link of this type exists.
            CANNOT_CREATE_APP_PRE_REGISTRATION_FOR_NON_ANDROID_APP (50):
                The app pre-registration campaign cannot be
                created for non-Android applications.
            APP_NOT_AVAILABLE_TO_CREATE_APP_PRE_REGISTRATION_CAMPAIGN (51):
                The campaign cannot be created since the app
                is not available for pre-registration in any
                country.
            INCOMPATIBLE_BUDGET_TYPE (52):
                The type of the Budget is not compatible with
                this Campaign.
            LOCAL_SERVICES_DUPLICATE_CATEGORY_BID (53):
                Category bid list in the local services
                campaign setting contains multiple bids for the
                same category ID.
            LOCAL_SERVICES_INVALID_CATEGORY_BID (54):
                Category bid list in the local services
                campaign setting contains a bid for an invalid
                category ID.
            LOCAL_SERVICES_MISSING_CATEGORY_BID (55):
                Category bid list in the local services
                campaign setting is missing a bid for a category
                ID that must be present.
            INVALID_STATUS_CHANGE (57):
                The requested change in status is not
                supported.
            MISSING_TRAVEL_CUSTOMER_LINK (58):
                Travel Campaign's travel_account_id does not match any
                customer links.
            INVALID_TRAVEL_CUSTOMER_LINK (59):
                Travel Campaign's travel_account_id matches an existing
                customer link but the customer link is not active.
            INVALID_EXCLUDED_PARENT_ASSET_SET_TYPE (62):
                The asset set type is invalid to be set in
                excluded_parent_asset_set_types field.
            ASSET_SET_NOT_A_HOTEL_PROPERTY_ASSET_SET (63):
                Campaign.hotel_property_asset_set must point to an asset set
                of type HOTEL_PROPERTY.
            HOTEL_PROPERTY_ASSET_SET_ONLY_FOR_PERFORMANCE_MAX_FOR_TRAVEL_GOALS (64):
                The hotel property asset set can only be set
                on Performance Max for travel goals campaigns.
            AVERAGE_DAILY_SPEND_TOO_HIGH (65):
                Customer's average daily spend is too high to
                enable this feature.
            CANNOT_ATTACH_TO_REMOVED_CAMPAIGN_GROUP (66):
                Cannot attach the campaign to a deleted
                campaign group.
            CANNOT_ATTACH_TO_BIDDING_STRATEGY (67):
                Cannot attach the campaign to this bidding
                strategy.
            CANNOT_CHANGE_BUDGET_PERIOD (68):
                A budget with a different period cannot be
                assigned to the campaign.
            NOT_ENOUGH_CONVERSIONS (71):
                Customer does not have enough conversions to
                enable this feature.
            CANNOT_SET_MORE_THAN_ONE_CONVERSION_ACTION (72):
                This campaign type can only have one
                conversion action.
            NOT_COMPATIBLE_WITH_BUDGET_TYPE (73):
                The field is not compatible with the budget
                type.
            NOT_COMPATIBLE_WITH_UPLOAD_CLICKS_CONVERSION (74):
                The feature is incompatible with
                ConversionActionType.UPLOAD_CLICKS.
            APP_ID_MUST_MATCH_CONVERSION_ACTION_APP_ID (76):
                App campaign setting app ID must match
                selective optimization conversion action app ID.
            CONVERSION_ACTION_WITH_DOWNLOAD_CATEGORY_NOT_ALLOWED (77):
                Selective optimization conversion action with
                Download category is not allowed.
            CONVERSION_ACTION_WITH_DOWNLOAD_CATEGORY_REQUIRED (78):
                One software download for selective
                optimization conversion action is required for
                this campaign conversion action.
            CONVERSION_TRACKING_NOT_ENABLED (79):
                Conversion tracking is not enabled and is
                required for this feature.
            NOT_COMPATIBLE_WITH_BIDDING_STRATEGY_TYPE (80):
                The field is not compatible with the bidding
                strategy type.
            NOT_COMPATIBLE_WITH_GOOGLE_ATTRIBUTION_CONVERSIONS (81):
                Campaign is not compatible with a conversion
                tracker that has Google attribution enabled.
            CONVERSION_LAG_TOO_HIGH (82):
                Customer level conversion lag is too high.
            NOT_LINKED_ADVERTISING_PARTNER (83):
                The advertiser set as an advertising partner
                is not an actively linked advertiser to this
                customer.
            INVALID_NUMBER_OF_ADVERTISING_PARTNER_IDS (84):
                Invalid number of advertising partner IDs.
            CANNOT_TARGET_DISPLAY_NETWORK_WITHOUT_YOUTUBE (85):
                Cannot target the display network without
                also targeting YouTube.
            CANNOT_LINK_TO_COMPARISON_SHOPPING_SERVICE_ACCOUNT (86):
                This campaign type cannot be linked to a
                Comparison Shopping Service account.
            CANNOT_TARGET_NETWORK_FOR_COMPARISON_SHOPPING_SERVICE_LINKED_ACCOUNTS (87):
                Standard Shopping campaigns that are linked
                to a Comparison Shopping Service account cannot
                target this network.
            CANNOT_MODIFY_TEXT_ASSET_AUTOMATION_WITH_ENABLED_TRIAL (88):
                Text asset automation settings can not be
                modified when there is an active Performance Max
                optimization automatically created assets
                experiment. End the experiment to modify these
                settings.
            DYNAMIC_TEXT_ASSET_CANNOT_OPT_OUT_WITH_FINAL_URL_EXPANSION_OPT_IN (89):
                Dynamic text asset cannot be opted out when
                final URL expansion is opted in.
            CANNOT_SET_CAMPAIGN_KEYWORD_MATCH_TYPE (90):
                Can not set a campaign level match type.
            CANNOT_DISABLE_BROAD_MATCH_WHEN_KEYWORD_CONVERSION_IN_PROCESS (91):
                The campaign level keyword match type cannot
                be switched to non-broad when keyword conversion
                to broad match is in process.
            CANNOT_DISABLE_BROAD_MATCH_WHEN_TARGETING_BRANDS (92):
                The campaign level keyword match type cannot
                be switched to non-broad when the campaign has
                any attached brand list or when a brand hint
                shared set is attached to the campaign.
            CANNOT_ENABLE_BROAD_MATCH_FOR_BASE_CAMPAIGN_WITH_PROMOTING_TRIAL (93):
                Cannot set campaign level keyword match type
                to BROAD if the campaign is a base campaign with
                an associated trial that is currently promoting.
            CANNOT_ENABLE_BROAD_MATCH_FOR_PROMOTING_TRIAL_CAMPAIGN (94):
                Cannot set campaign level keyword match type
                to BROAD if the campaign is a trial currently
                promoting.
            REQUIRED_BUSINESS_NAME_ASSET_NOT_LINKED (95):
                Performance Max campaigns with Brand
                Guidelines enabled require at least one business
                name to be linked as a CampaignAsset.
                Performance Max campaigns for online sales with
                a product feed must meet this requirement only
                when there are assets that are linked to the
                campaign's asset groups.
            REQUIRED_LOGO_ASSET_NOT_LINKED (96):
                Performance Max campaigns with Brand
                Guidelines enabled require at least one square
                logo to be linked as a CampaignAsset.
                Performance Max campaigns for online sales with
                a product feed must meet this requirement only
                when there are assets that are linked to the
                campaign's asset groups.
            BRAND_TARGETING_OVERRIDES_NOT_SUPPORTED (97):
                This campaign does not support brand
                targeting overrides. Brand targeting overrides
                are only supported for Performance Max campaigns
                that have a product feed.
            BRAND_GUIDELINES_NOT_ENABLED_FOR_CAMPAIGN (98):
                Brand Guideline fields can only be set for
                campaigns that have Brand Guidelines enabled.
            BRAND_GUIDELINES_MAIN_AND_ACCENT_COLORS_REQUIRED (99):
                When a Brand Guidelines color field is set,
                both main color and accent color are required.
            BRAND_GUIDELINES_COLOR_INVALID_FORMAT (100):
                Brand Guidelines colors must be hex colors matching the
                regular expression '#[0-9a-fA-F]{6}', for example '#abc123'
            BRAND_GUIDELINES_UNSUPPORTED_FONT_FAMILY (101):
                Brand Guidelines font family must be one of the supported
                Google Fonts. See
                Campaign.brand_guidelines.predefined_font_family for the
                list of supported fonts.
            BRAND_GUIDELINES_UNSUPPORTED_CHANNEL (102):
                Brand Guidelines cannot be set for this
                channel type. Brand Guidelines supports
                Performance Max campaigns.
            CANNOT_ENABLE_BRAND_GUIDELINES_FOR_TRAVEL_GOALS (103):
                Brand Guidelines cannot be enabled for
                Performance Max for travel goals campaigns.
            CUSTOMER_NOT_ALLOWLISTED_FOR_BRAND_GUIDELINES (104):
                This customer is not allowlisted for enabling
                Brand Guidelines.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_TARGET_CONTENT_NETWORK = 3
        CANNOT_TARGET_SEARCH_NETWORK = 4
        CANNOT_TARGET_SEARCH_NETWORK_WITHOUT_GOOGLE_SEARCH = 5
        CANNOT_TARGET_GOOGLE_SEARCH_FOR_CPM_CAMPAIGN = 6
        CAMPAIGN_MUST_TARGET_AT_LEAST_ONE_NETWORK = 7
        CANNOT_TARGET_PARTNER_SEARCH_NETWORK = 8
        CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CRITERIA_LEVEL_BIDDING_STRATEGY = (
            9
        )
        CAMPAIGN_DURATION_MUST_CONTAIN_ALL_RUNNABLE_TRIALS = 10
        CANNOT_MODIFY_FOR_TRIAL_CAMPAIGN = 11
        DUPLICATE_CAMPAIGN_NAME = 12
        INCOMPATIBLE_CAMPAIGN_FIELD = 13
        INVALID_CAMPAIGN_NAME = 14
        INVALID_AD_SERVING_OPTIMIZATION_STATUS = 15
        INVALID_TRACKING_URL = 16
        CANNOT_SET_BOTH_TRACKING_URL_TEMPLATE_AND_TRACKING_SETTING = 17
        MAX_IMPRESSIONS_NOT_IN_RANGE = 18
        TIME_UNIT_NOT_SUPPORTED = 19
        INVALID_OPERATION_IF_SERVING_STATUS_HAS_ENDED = 20
        BUDGET_CANNOT_BE_SHARED = 21
        CAMPAIGN_CANNOT_USE_SHARED_BUDGET = 22
        CANNOT_CHANGE_BUDGET_ON_CAMPAIGN_WITH_TRIALS = 23
        CAMPAIGN_LABEL_DOES_NOT_EXIST = 24
        CAMPAIGN_LABEL_ALREADY_EXISTS = 25
        MISSING_SHOPPING_SETTING = 26
        INVALID_SHOPPING_SALES_COUNTRY = 27
        ADVERTISING_CHANNEL_TYPE_NOT_AVAILABLE_FOR_ACCOUNT_TYPE = 31
        INVALID_ADVERTISING_CHANNEL_SUB_TYPE = 32
        AT_LEAST_ONE_CONVERSION_MUST_BE_SELECTED = 33
        CANNOT_SET_AD_ROTATION_MODE = 34
        CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED = 35
        CANNOT_SET_DATE_TO_PAST = 36
        MISSING_HOTEL_CUSTOMER_LINK = 37
        INVALID_HOTEL_CUSTOMER_LINK = 38
        MISSING_HOTEL_SETTING = 39
        CANNOT_USE_SHARED_CAMPAIGN_BUDGET_WHILE_PART_OF_CAMPAIGN_GROUP = 40
        APP_NOT_FOUND = 41
        SHOPPING_ENABLE_LOCAL_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE = 42
        MERCHANT_NOT_ALLOWED_FOR_COMPARISON_LISTING_ADS = 43
        INSUFFICIENT_APP_INSTALLS_COUNT = 44
        SENSITIVE_CATEGORY_APP = 45
        HEC_AGREEMENT_REQUIRED = 46
        NOT_COMPATIBLE_WITH_VIEW_THROUGH_CONVERSION_OPTIMIZATION = 49
        INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE = 48
        CANNOT_CREATE_APP_PRE_REGISTRATION_FOR_NON_ANDROID_APP = 50
        APP_NOT_AVAILABLE_TO_CREATE_APP_PRE_REGISTRATION_CAMPAIGN = 51
        INCOMPATIBLE_BUDGET_TYPE = 52
        LOCAL_SERVICES_DUPLICATE_CATEGORY_BID = 53
        LOCAL_SERVICES_INVALID_CATEGORY_BID = 54
        LOCAL_SERVICES_MISSING_CATEGORY_BID = 55
        INVALID_STATUS_CHANGE = 57
        MISSING_TRAVEL_CUSTOMER_LINK = 58
        INVALID_TRAVEL_CUSTOMER_LINK = 59
        INVALID_EXCLUDED_PARENT_ASSET_SET_TYPE = 62
        ASSET_SET_NOT_A_HOTEL_PROPERTY_ASSET_SET = 63
        HOTEL_PROPERTY_ASSET_SET_ONLY_FOR_PERFORMANCE_MAX_FOR_TRAVEL_GOALS = 64
        AVERAGE_DAILY_SPEND_TOO_HIGH = 65
        CANNOT_ATTACH_TO_REMOVED_CAMPAIGN_GROUP = 66
        CANNOT_ATTACH_TO_BIDDING_STRATEGY = 67
        CANNOT_CHANGE_BUDGET_PERIOD = 68
        NOT_ENOUGH_CONVERSIONS = 71
        CANNOT_SET_MORE_THAN_ONE_CONVERSION_ACTION = 72
        NOT_COMPATIBLE_WITH_BUDGET_TYPE = 73
        NOT_COMPATIBLE_WITH_UPLOAD_CLICKS_CONVERSION = 74
        APP_ID_MUST_MATCH_CONVERSION_ACTION_APP_ID = 76
        CONVERSION_ACTION_WITH_DOWNLOAD_CATEGORY_NOT_ALLOWED = 77
        CONVERSION_ACTION_WITH_DOWNLOAD_CATEGORY_REQUIRED = 78
        CONVERSION_TRACKING_NOT_ENABLED = 79
        NOT_COMPATIBLE_WITH_BIDDING_STRATEGY_TYPE = 80
        NOT_COMPATIBLE_WITH_GOOGLE_ATTRIBUTION_CONVERSIONS = 81
        CONVERSION_LAG_TOO_HIGH = 82
        NOT_LINKED_ADVERTISING_PARTNER = 83
        INVALID_NUMBER_OF_ADVERTISING_PARTNER_IDS = 84
        CANNOT_TARGET_DISPLAY_NETWORK_WITHOUT_YOUTUBE = 85
        CANNOT_LINK_TO_COMPARISON_SHOPPING_SERVICE_ACCOUNT = 86
        CANNOT_TARGET_NETWORK_FOR_COMPARISON_SHOPPING_SERVICE_LINKED_ACCOUNTS = (
            87
        )
        CANNOT_MODIFY_TEXT_ASSET_AUTOMATION_WITH_ENABLED_TRIAL = 88
        DYNAMIC_TEXT_ASSET_CANNOT_OPT_OUT_WITH_FINAL_URL_EXPANSION_OPT_IN = 89
        CANNOT_SET_CAMPAIGN_KEYWORD_MATCH_TYPE = 90
        CANNOT_DISABLE_BROAD_MATCH_WHEN_KEYWORD_CONVERSION_IN_PROCESS = 91
        CANNOT_DISABLE_BROAD_MATCH_WHEN_TARGETING_BRANDS = 92
        CANNOT_ENABLE_BROAD_MATCH_FOR_BASE_CAMPAIGN_WITH_PROMOTING_TRIAL = 93
        CANNOT_ENABLE_BROAD_MATCH_FOR_PROMOTING_TRIAL_CAMPAIGN = 94
        REQUIRED_BUSINESS_NAME_ASSET_NOT_LINKED = 95
        REQUIRED_LOGO_ASSET_NOT_LINKED = 96
        BRAND_TARGETING_OVERRIDES_NOT_SUPPORTED = 97
        BRAND_GUIDELINES_NOT_ENABLED_FOR_CAMPAIGN = 98
        BRAND_GUIDELINES_MAIN_AND_ACCENT_COLORS_REQUIRED = 99
        BRAND_GUIDELINES_COLOR_INVALID_FORMAT = 100
        BRAND_GUIDELINES_UNSUPPORTED_FONT_FAMILY = 101
        BRAND_GUIDELINES_UNSUPPORTED_CHANNEL = 102
        CANNOT_ENABLE_BRAND_GUIDELINES_FOR_TRAVEL_GOALS = 103
        CUSTOMER_NOT_ALLOWLISTED_FOR_BRAND_GUIDELINES = 104


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A gRPC Interceptor that is responsible for logging requests and responses.

This class is initialized in the GoogleAdsClient and passed into a grpc
intercept_channel whenever a new service is initialized. It intercepts requests
and responses, parses them into a human readable structure and logs them using
the passed in logger instance.
"""

from copy import deepcopy
from types import SimpleNamespace
import json
import logging

from grpc import UnaryUnaryClientInterceptor, UnaryStreamClientInterceptor

from google.ads.googleads.interceptors import Interceptor, mask_message
from types import SimpleNamespace


class LoggingInterceptor(
    Interceptor, UnaryUnaryClientInterceptor, UnaryStreamClientInterceptor
):
    """An interceptor that logs rpc requests and responses."""

    _FULL_REQUEST_LOG_LINE = (
        "Request\n-------\nMethod: {}\nHost: {}\n"
        "Headers: {}\nRequest: {}\n\nResponse\n-------\n"
        "Headers: {}\nResponse: {}\n"
    )
    _FULL_FAULT_LOG_LINE = (
        "Request\n-------\nMethod: {}\nHost: {}\n"
        "Headers: {}\nRequest: {}\n\nResponse\n-------\n"
        "Headers: {}\nFault: {}\n"
    )
    _SUMMARY_LOG_LINE = (
        "Request made: ClientCustomerId: {}, Host: {}, "
        "Method: {}, RequestId: {}, IsFault: {}, "
        "FaultMessage: {}"
    )

    def __init__(self, logger, api_version, endpoint=None):
        """Initializer for the LoggingInterceptor.

        Args:
            logger: An instance of logging.Logger.
            api_version: a str of the API version of the request.
            endpoint: a str specifying the endpoint for requests.
        """
        super().__init__(api_version)
        self.endpoint = endpoint
        self.logger = logger
        self._cache = None

    def _get_trailing_metadata(self, response):
        """Retrieves trailing metadata from a response object.

        If the exception is a GoogleAdsException the trailing metadata will be
        on its error object, otherwise it will be on the response object.

        Returns:
            A tuple of metadatum representing response header key value pairs.

        Args:
            response: A grpc.Call/grpc.Future instance.
        """
        try:
            trailing_metadata = response.trailing_metadata()

            if not trailing_metadata:
                return self.get_trailing_metadata_from_interceptor_exception(
                    response.exception()
                )

            return trailing_metadata
        except AttributeError:
            return self.get_trailing_metadata_from_interceptor_exception(
                response.exception()
            )

    def _get_initial_metadata(self, client_call_details):
        """Retrieves the initial metadata from client_call_details.

        Returns an empty tuple if metadata isn't present on the
        client_call_details object.

        Returns:
            A tuple of metadatum representing request header key value pairs.

        Args:
            client_call_details: An instance of grpc.ClientCallDetails.
        """
        return getattr(client_call_details, "metadata", tuple())

    def _get_call_method(self, client_call_details):
        """Retrieves the call method from client_call_details.

        Returns None if the method is not present on the client_call_details
        object.

        Returns:
            A str with the call method or None if it isn't present.

        Args:
            client_call_details: An instance of grpc.ClientCallDetails.
        """
        return getattr(client_call_details, "method", None)

    def _get_customer_id(self, request):
        """Retrieves the customer_id from the grpc request.

        Returns None if a customer_id is not present on the request object.

        Returns:
            A str with the customer id from the request or None if it isn't
            present.

        Args:
            request: An instance of a request proto message.
        """
        if hasattr(request, "customer_id"):
            return getattr(request, "customer_id")
        elif hasattr(request, "resource_name"):
            resource_name = getattr(request, "resource_name")
            segments = resource_name.split("/")
            if segments[0] == "customers":
                return segments[1]
        else:
            return None

    def _parse_exception_to_str(self, exception):
        """Parses response exception object to str for logging.

        Returns:
            A str representing a exception from the API.

        Args:
            exception: A grpc.Call instance.
        """
        try:
            # If the exception is from the Google Ads API then the failure
            # attribute will be an instance of GoogleAdsFailure. We convert it
            # to str here so the return type is consistent, rather than casting
            # by concatenating in the logs.
            return str(exception.failure)
        except AttributeError:
            try:
                # if exception.failure isn't present then it's likely this is a
                # transport error with a .debug_error_string method and the
                # returned JSON string will need to be formatted.
                return self.format_json_object(
                    json.loads(exception.debug_error_string())
                )
            except (AttributeError, ValueError):
                # if both attempts to retrieve serializable error data fail
                # then simply return an empty JSON string
                return "{}"

    def _get_fault_message(self, exception):
        """Retrieves a fault/error message from an exception object.

        Returns None if no error message can be found on the exception.

        Returns:
            A str with an error message or None if one cannot be found.

        Args:
            response: A grpc.Call/grpc.Future instance.
            exception: A grpc.Call instance.
        """
        try:
            return exception.failure.errors[0].message
        except AttributeError:
            try:
                return exception.details()
            except AttributeError:
                return None

    def log_successful_request(
        self,
        method,
        customer_id,
        metadata_json,
        request_id,
        request,
        trailing_metadata_json,
        response,
    ):
        """Handles logging of a successful request.

        Args:
            method: The method of the request.
            customer_id: The customer ID associated with the request.
            metadata_json: A JSON str of initial_metadata.
            request_id: A unique ID for the request provided in the response.
            request: An instance of a request proto message.
            trailing_metadata_json: A JSON str of trailing_metadata.
            response: A grpc.Call/grpc.Future instance.
        """
        result = self.retrieve_and_mask_result(response)

        # Check log level here to avoid calling .format() unless necessary.
        if self.logger.isEnabledFor(logging.DEBUG):
            self.logger.debug(
                self._FULL_REQUEST_LOG_LINE.format(
                    method,
                    self.endpoint,
                    metadata_json,
                    request,
                    trailing_metadata_json,
                    result,
                )
            )

        if self.logger.isEnabledFor(logging.INFO):
            self.logger.info(
                self._SUMMARY_LOG_LINE.format(
                    customer_id, self.endpoint, method, request_id, False, None
                )
            )

    def log_failed_request(
        self,
        method,
        customer_id,
        metadata_json,
        request_id,
        request,
        trailing_metadata_json,
        response,
    ):
        """Handles logging of a failed request.

        Args:
            method: The method of the request.
            customer_id: The customer ID associated with the request.
            metadata_json: A JSON str of initial_metadata.
            request_id: A unique ID for the request provided in the response.
            request: An instance of a request proto message.
            trailing_metadata_json: A JSON str of trailing_metadata.
            response: A JSON str of the response message.
        """
        exception = self._get_error_from_response(response)
        exception_str = self._parse_exception_to_str(exception)
        fault_message = self._get_fault_message(exception)

        if self.logger.isEnabledFor(logging.INFO):
            self.logger.info(
                self._FULL_FAULT_LOG_LINE.format(
                    method,
                    self.endpoint,
                    metadata_json,
                    request,
                    trailing_metadata_json,
                    exception_str,
                )
            )

        self.logger.warning(
            self._SUMMARY_LOG_LINE.format(
                customer_id,
                self.endpoint,
                method,
                request_id,
                True,
                fault_message,
            )
        )

    def log_request(self, client_call_details, request, response):
        """Handles logging all requests.

        Args:
            client_call_details: An instance of grpc.ClientCallDetails.
            request: An instance of a request proto message.
            response: A grpc.Call/grpc.Future instance.
        """
        method = self._get_call_method(client_call_details)
        customer_id = self._get_customer_id(request)
        initial_metadata = self._get_initial_metadata(client_call_details)
        initial_metadata_json = self.parse_metadata_to_json(initial_metadata)
        trailing_metadata = self._get_trailing_metadata(response)
        request_id = self.get_request_id_from_metadata(trailing_metadata)
        trailing_metadata_json = self.parse_metadata_to_json(trailing_metadata)
        request = mask_message(request, self._SENSITIVE_INFO_MASK)

        if response.exception():
            self.log_failed_request(
                method,
                customer_id,
                initial_metadata_json,
                request_id,
                request,
                trailing_metadata_json,
                response,
            )
        else:
            self.log_successful_request(
                method,
                customer_id,
                initial_metadata_json,
                request_id,
                request,
                trailing_metadata_json,
                response,
            )

    def intercept_unary_unary(self, continuation, client_call_details, request):
        """Intercepts and logs API interactions.

        Overrides abstract method defined in grpc.UnaryUnaryClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """
        response = continuation(client_call_details, request)

        if self.logger.isEnabledFor(logging.WARNING):
            self.log_request(client_call_details, request, response)

        return response

    def intercept_unary_stream(
        self, continuation, client_call_details, request
    ):
        """Intercepts and logs API interactions for Unary-Stream requests.

        Overrides abstract method defined in grpc.UnaryStreamClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """

        def on_rpc_complete(response_future):
            if self.logger.isEnabledFor(logging.WARNING):
                self.log_request(client_call_details, request, response_future)

        response = continuation(client_call_details, request)

        response.add_done_callback(on_rpc_complete)
        self._cache = response.get_cache()
        return response

    def retrieve_and_mask_result(self, response):
        """If the cache is populated, mask the cached stream response object.
        Otherwise, mask the non-streaming response result.

        Args:
            response: A grpc.Call/grpc.Future instance.

        Returns:
            A masked response message.
        """
        if self._cache and self._cache.initial_response_object is not None:
            # Get the first streaming response object from the cache.
            result = self._cache.initial_response_object
        else:
            result = response.result()
        # Mask the response object if debug level logging is enabled.
        if type(self.logger) == logging.Logger and self.logger.isEnabledFor(
            logging.DEBUG
        ):
            result = mask_message(result, self._SENSITIVE_INFO_MASK)

        return result



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import change_client_type
from google.ads.googleads.v22.enums.types import change_event_resource_type
from google.ads.googleads.v22.enums.types import (
    resource_change_operation as gage_resource_change_operation,
)
from google.ads.googleads.v22.resources.types import ad as gagr_ad
from google.ads.googleads.v22.resources.types import ad_group as gagr_ad_group
from google.ads.googleads.v22.resources.types import (
    ad_group_ad as gagr_ad_group_ad,
)
from google.ads.googleads.v22.resources.types import (
    ad_group_asset as gagr_ad_group_asset,
)
from google.ads.googleads.v22.resources.types import (
    ad_group_bid_modifier as gagr_ad_group_bid_modifier,
)
from google.ads.googleads.v22.resources.types import (
    ad_group_criterion as gagr_ad_group_criterion,
)
from google.ads.googleads.v22.resources.types import asset as gagr_asset
from google.ads.googleads.v22.resources.types import asset_set as gagr_asset_set
from google.ads.googleads.v22.resources.types import (
    asset_set_asset as gagr_asset_set_asset,
)
from google.ads.googleads.v22.resources.types import campaign as gagr_campaign
from google.ads.googleads.v22.resources.types import (
    campaign_asset as gagr_campaign_asset,
)
from google.ads.googleads.v22.resources.types import (
    campaign_asset_set as gagr_campaign_asset_set,
)
from google.ads.googleads.v22.resources.types import (
    campaign_budget as gagr_campaign_budget,
)
from google.ads.googleads.v22.resources.types import (
    campaign_criterion as gagr_campaign_criterion,
)
from google.ads.googleads.v22.resources.types import (
    customer_asset as gagr_customer_asset,
)
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "ChangeEvent",
    },
)


class ChangeEvent(proto.Message):
    r"""Describes the granular change of returned resources of
    certain resource types. Changes made through the UI or API in
    the past 30 days are included. Previous and new values of the
    changed fields are shown. ChangeEvent could have up to 3 minutes
    delay to reflect a new change.

    Attributes:
        resource_name (str):
            Output only. The resource name of the change event. Change
            event resource names have the form:

            ``customers/{customer_id}/changeEvents/{timestamp_micros}~{command_index}~{mutate_index}``
        change_date_time (str):
            Output only. Time at which the change was
            committed on this resource.
        change_resource_type (google.ads.googleads.v22.enums.types.ChangeEventResourceTypeEnum.ChangeEventResourceType):
            Output only. The type of the changed resource. This dictates
            what resource will be set in old_resource and new_resource.
        change_resource_name (str):
            Output only. The Simply resource this change
            occurred on.
        client_type (google.ads.googleads.v22.enums.types.ChangeClientTypeEnum.ChangeClientType):
            Output only. Where the change was made
            through.
        user_email (str):
            Output only. The email of the user who made
            this change.
        old_resource (google.ads.googleads.v22.resources.types.ChangeEvent.ChangedResource):
            Output only. The old resource before the
            change. Only changed fields will be populated.
        new_resource (google.ads.googleads.v22.resources.types.ChangeEvent.ChangedResource):
            Output only. The new resource after the
            change. Only changed fields will be populated.
        resource_change_operation (google.ads.googleads.v22.enums.types.ResourceChangeOperationEnum.ResourceChangeOperation):
            Output only. The operation on the changed
            resource.
        changed_fields (google.protobuf.field_mask_pb2.FieldMask):
            Output only. A list of fields that are
            changed in the returned resource.
        campaign (str):
            Output only. The Campaign affected by this
            change.
        ad_group (str):
            Output only. The AdGroup affected by this
            change.
        asset (str):
            Output only. The Asset affected by this
            change.
    """

    class ChangedResource(proto.Message):
        r"""A wrapper proto presenting all supported resources. Only the
        resource of the change_resource_type will be set.

        Attributes:
            ad (google.ads.googleads.v22.resources.types.Ad):
                Output only. Set if change_resource_type == AD.
            ad_group (google.ads.googleads.v22.resources.types.AdGroup):
                Output only. Set if change_resource_type == AD_GROUP.
            ad_group_criterion (google.ads.googleads.v22.resources.types.AdGroupCriterion):
                Output only. Set if change_resource_type ==
                AD_GROUP_CRITERION.
            campaign (google.ads.googleads.v22.resources.types.Campaign):
                Output only. Set if change_resource_type == CAMPAIGN.
            campaign_budget (google.ads.googleads.v22.resources.types.CampaignBudget):
                Output only. Set if change_resource_type == CAMPAIGN_BUDGET.
            ad_group_bid_modifier (google.ads.googleads.v22.resources.types.AdGroupBidModifier):
                Output only. Set if change_resource_type ==
                AD_GROUP_BID_MODIFIER.
            campaign_criterion (google.ads.googleads.v22.resources.types.CampaignCriterion):
                Output only. Set if change_resource_type ==
                CAMPAIGN_CRITERION.
            ad_group_ad (google.ads.googleads.v22.resources.types.AdGroupAd):
                Output only. Set if change_resource_type == AD_GROUP_AD.
            asset (google.ads.googleads.v22.resources.types.Asset):
                Output only. Set if change_resource_type == ASSET.
            customer_asset (google.ads.googleads.v22.resources.types.CustomerAsset):
                Output only. Set if change_resource_type == CUSTOMER_ASSET.
            campaign_asset (google.ads.googleads.v22.resources.types.CampaignAsset):
                Output only. Set if change_resource_type == CAMPAIGN_ASSET.
            ad_group_asset (google.ads.googleads.v22.resources.types.AdGroupAsset):
                Output only. Set if change_resource_type == AD_GROUP_ASSET.
            asset_set (google.ads.googleads.v22.resources.types.AssetSet):
                Output only. Set if change_resource_type == ASSET_SET.
            asset_set_asset (google.ads.googleads.v22.resources.types.AssetSetAsset):
                Output only. Set if change_resource_type == ASSET_SET_ASSET.
            campaign_asset_set (google.ads.googleads.v22.resources.types.CampaignAssetSet):
                Output only. Set if change_resource_type ==
                CAMPAIGN_ASSET_SET.
        """

        ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=1,
            message=gagr_ad.Ad,
        )
        ad_group: gagr_ad_group.AdGroup = proto.Field(
            proto.MESSAGE,
            number=2,
            message=gagr_ad_group.AdGroup,
        )
        ad_group_criterion: gagr_ad_group_criterion.AdGroupCriterion = (
            proto.Field(
                proto.MESSAGE,
                number=3,
                message=gagr_ad_group_criterion.AdGroupCriterion,
            )
        )
        campaign: gagr_campaign.Campaign = proto.Field(
            proto.MESSAGE,
            number=4,
            message=gagr_campaign.Campaign,
        )
        campaign_budget: gagr_campaign_budget.CampaignBudget = proto.Field(
            proto.MESSAGE,
            number=5,
            message=gagr_campaign_budget.CampaignBudget,
        )
        ad_group_bid_modifier: gagr_ad_group_bid_modifier.AdGroupBidModifier = (
            proto.Field(
                proto.MESSAGE,
                number=6,
                message=gagr_ad_group_bid_modifier.AdGroupBidModifier,
            )
        )
        campaign_criterion: gagr_campaign_criterion.CampaignCriterion = (
            proto.Field(
                proto.MESSAGE,
                number=7,
                message=gagr_campaign_criterion.CampaignCriterion,
            )
        )
        ad_group_ad: gagr_ad_group_ad.AdGroupAd = proto.Field(
            proto.MESSAGE,
            number=12,
            message=gagr_ad_group_ad.AdGroupAd,
        )
        asset: gagr_asset.Asset = proto.Field(
            proto.MESSAGE,
            number=13,
            message=gagr_asset.Asset,
        )
        customer_asset: gagr_customer_asset.CustomerAsset = proto.Field(
            proto.MESSAGE,
            number=14,
            message=gagr_customer_asset.CustomerAsset,
        )
        campaign_asset: gagr_campaign_asset.CampaignAsset = proto.Field(
            proto.MESSAGE,
            number=15,
            message=gagr_campaign_asset.CampaignAsset,
        )
        ad_group_asset: gagr_ad_group_asset.AdGroupAsset = proto.Field(
            proto.MESSAGE,
            number=16,
            message=gagr_ad_group_asset.AdGroupAsset,
        )
        asset_set: gagr_asset_set.AssetSet = proto.Field(
            proto.MESSAGE,
            number=17,
            message=gagr_asset_set.AssetSet,
        )
        asset_set_asset: gagr_asset_set_asset.AssetSetAsset = proto.Field(
            proto.MESSAGE,
            number=18,
            message=gagr_asset_set_asset.AssetSetAsset,
        )
        campaign_asset_set: gagr_campaign_asset_set.CampaignAssetSet = (
            proto.Field(
                proto.MESSAGE,
                number=19,
                message=gagr_campaign_asset_set.CampaignAssetSet,
            )
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    change_date_time: str = proto.Field(
        proto.STRING,
        number=2,
    )
    change_resource_type: (
        change_event_resource_type.ChangeEventResourceTypeEnum.ChangeEventResourceType
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=change_event_resource_type.ChangeEventResourceTypeEnum.ChangeEventResourceType,
    )
    change_resource_name: str = proto.Field(
        proto.STRING,
        number=4,
    )
    client_type: change_client_type.ChangeClientTypeEnum.ChangeClientType = (
        proto.Field(
            proto.ENUM,
            number=5,
            enum=change_client_type.ChangeClientTypeEnum.ChangeClientType,
        )
    )
    user_email: str = proto.Field(
        proto.STRING,
        number=6,
    )
    old_resource: ChangedResource = proto.Field(
        proto.MESSAGE,
        number=7,
        message=ChangedResource,
    )
    new_resource: ChangedResource = proto.Field(
        proto.MESSAGE,
        number=8,
        message=ChangedResource,
    )
    resource_change_operation: (
        gage_resource_change_operation.ResourceChangeOperationEnum.ResourceChangeOperation
    ) = proto.Field(
        proto.ENUM,
        number=9,
        enum=gage_resource_change_operation.ResourceChangeOperationEnum.ResourceChangeOperation,
    )
    changed_fields: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=10,
        message=field_mask_pb2.FieldMask,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=11,
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=12,
    )
    asset: str = proto.Field(
        proto.STRING,
        number=20,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import additional_application_info
from google.ads.googleads.v21.common.types import audience_insights_attribute
from google.ads.googleads.v21.common.types import criteria
from google.ads.googleads.v21.common.types import dates
from google.ads.googleads.v21.enums.types import audience_insights_dimension
from google.ads.googleads.v21.enums.types import (
    audience_insights_marketing_objective,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "GenerateInsightsFinderReportRequest",
        "GenerateInsightsFinderReportResponse",
        "GenerateAudienceCompositionInsightsRequest",
        "GenerateAudienceCompositionInsightsResponse",
        "GenerateSuggestedTargetingInsightsRequest",
        "GenerateSuggestedTargetingInsightsResponse",
        "TargetingSuggestionMetrics",
        "ListAudienceInsightsAttributesRequest",
        "ListAudienceInsightsAttributesResponse",
        "ListInsightsEligibleDatesRequest",
        "ListInsightsEligibleDatesResponse",
        "GenerateAudienceOverlapInsightsRequest",
        "GenerateAudienceOverlapInsightsResponse",
        "DimensionOverlapResult",
        "AudienceOverlapItem",
        "GenerateTargetingSuggestionMetricsRequest",
        "GenerateTargetingSuggestionMetricsResponse",
        "BasicInsightsAudience",
        "InsightsAudienceDefinition",
        "InsightsAudienceDescription",
        "InsightsAudience",
        "InsightsAudienceAttributeGroup",
        "AudienceCompositionSection",
        "AudienceCompositionAttributeCluster",
        "AudienceCompositionMetrics",
        "AudienceCompositionAttribute",
    },
)


class GenerateInsightsFinderReportRequest(proto.Message):
    r"""Request message for
    [AudienceInsightsService.GenerateInsightsFinderReport][google.ads.googleads.v21.services.AudienceInsightsService.GenerateInsightsFinderReport].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        baseline_audience (google.ads.googleads.v21.services.types.BasicInsightsAudience):
            Required. A baseline audience for this
            report, typically all people in a region.
        specific_audience (google.ads.googleads.v21.services.types.BasicInsightsAudience):
            Required. The specific audience of interest
            for this report.  The insights in the report
            will be based on attributes more prevalent in
            this audience than in the report's baseline
            audience.
        customer_insights_group (str):
            The name of the customer being planned for.
            This is a user-defined value.
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    baseline_audience: "BasicInsightsAudience" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="BasicInsightsAudience",
    )
    specific_audience: "BasicInsightsAudience" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="BasicInsightsAudience",
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=4,
    )
    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=5,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class GenerateInsightsFinderReportResponse(proto.Message):
    r"""The response message for
    [AudienceInsightsService.GenerateInsightsFinderReport][google.ads.googleads.v21.services.AudienceInsightsService.GenerateInsightsFinderReport],
    containing the shareable URL for the report.

    Attributes:
        saved_report_url (str):
            An HTTPS URL providing a deep link into the
            Insights Finder UI with the report inputs filled
            in according to the request.
    """

    saved_report_url: str = proto.Field(
        proto.STRING,
        number=1,
    )


class GenerateAudienceCompositionInsightsRequest(proto.Message):
    r"""Request message for
    [AudienceInsightsService.GenerateAudienceCompositionInsights][google.ads.googleads.v21.services.AudienceInsightsService.GenerateAudienceCompositionInsights].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        audience (google.ads.googleads.v21.services.types.InsightsAudience):
            Required. The audience of interest for which
            insights are being requested.
        baseline_audience (google.ads.googleads.v21.services.types.InsightsAudience):
            The baseline audience to which the audience
            of interest is being compared.
        data_month (str):
            The one-month range of historical data to use
            for insights, in the format "yyyy-mm". If unset,
            insights will be returned for the last thirty
            days of data.
        dimensions (MutableSequence[google.ads.googleads.v21.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension]):
            Required. The audience dimensions for which composition
            insights should be returned. Supported dimensions are
            KNOWLEDGE_GRAPH, GEO_TARGET_COUNTRY, SUB_COUNTRY_LOCATION,
            YOUTUBE_CHANNEL, YOUTUBE_LINEUP, AFFINITY_USER_INTEREST,
            IN_MARKET_USER_INTEREST, PARENTAL_STATUS, INCOME_RANGE,
            AGE_RANGE, and GENDER.
        customer_insights_group (str):
            The name of the customer being planned for.
            This is a user-defined value.
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    audience: "InsightsAudience" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="InsightsAudience",
    )
    baseline_audience: "InsightsAudience" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="InsightsAudience",
    )
    data_month: str = proto.Field(
        proto.STRING,
        number=3,
    )
    dimensions: MutableSequence[
        audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
    ] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum=audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension,
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=5,
    )
    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=7,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class GenerateAudienceCompositionInsightsResponse(proto.Message):
    r"""Response message for
    [AudienceInsightsService.GenerateAudienceCompositionInsights][google.ads.googleads.v21.services.AudienceInsightsService.GenerateAudienceCompositionInsights].

    Attributes:
        sections (MutableSequence[google.ads.googleads.v21.services.types.AudienceCompositionSection]):
            The contents of the insights report,
            organized into sections. Each section is
            associated with one of the
            AudienceInsightsDimension values in the request.
            There may be more than one section per
            dimension.
    """

    sections: MutableSequence["AudienceCompositionSection"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="AudienceCompositionSection",
        )
    )


class GenerateSuggestedTargetingInsightsRequest(proto.Message):
    r"""Request message for
    [AudienceInsightsService.GenerateSuggestedTargetingInsights][google.ads.googleads.v21.services.AudienceInsightsService.GenerateSuggestedTargetingInsights].

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        customer_insights_group (str):
            Optional. The name of the customer being
            planned for.  This is a user-defined value.
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
        audience_definition (google.ads.googleads.v21.services.types.InsightsAudienceDefinition):
            Provide a seed audience to get suggestions
            for.

            This field is a member of `oneof`_ ``audience_input``.
        audience_description (google.ads.googleads.v21.services.types.InsightsAudienceDescription):
            Provide a text description of an audience to
            get AI-generated targeting suggestions. This can
            take around 5 or more seconds to complete.

            This field is a member of `oneof`_ ``audience_input``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=5,
    )
    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=8,
        message=additional_application_info.AdditionalApplicationInfo,
    )
    audience_definition: "InsightsAudienceDefinition" = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="audience_input",
        message="InsightsAudienceDefinition",
    )
    audience_description: "InsightsAudienceDescription" = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="audience_input",
        message="InsightsAudienceDescription",
    )


class GenerateSuggestedTargetingInsightsResponse(proto.Message):
    r"""Response message for
    [AudienceInsightsService.GenerateSuggestedTargetingInsights][google.ads.googleads.v21.services.AudienceInsightsService.GenerateSuggestedTargetingInsights].

    Attributes:
        suggestions (MutableSequence[google.ads.googleads.v21.services.types.TargetingSuggestionMetrics]):
            Suggested insights for targetable audiences.
    """

    suggestions: MutableSequence["TargetingSuggestionMetrics"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="TargetingSuggestionMetrics",
        )
    )


class TargetingSuggestionMetrics(proto.Message):
    r"""A suggested targetable audience relevant to the requested
    audience.

    Attributes:
        locations (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata]):
            Suggested location targeting. These attributes all have
            dimension GEO_TARGET_COUNTRY or SUB_COUNTRY_LOCATION
        age_ranges (MutableSequence[google.ads.googleads.v21.common.types.AgeRangeInfo]):
            Suggested age targeting; may be empty
            indicating no age targeting.
        gender (google.ads.googleads.v21.common.types.GenderInfo):
            Suggested gender targeting.  If present, this
            attribute has dimension GENDER.
        parental_status (google.ads.googleads.v21.common.types.ParentalStatusInfo):
            A Parental Status value (parent, or not a
            parent).
        user_interests (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadataGroup]):
            List of user interest attributes with
            metadata defining the audience. The combination
            has a logical AND-of-ORs structure: The
            attributes within each
            AudienceInsightsAttributeMetadataGroup are ORed,
            and the groups themselves are ANDed.
        coverage (float):
            The fraction (from 0 to 1 inclusive) of the
            requested audience that can be reached using the
            suggested targeting.
        index (float):
            The ratio of coverage to the coverage of the
            baseline audience or zero if this ratio is
            undefined or is not meaningful.
        potential_youtube_reach (int):
            The approximate estimated number of people
            that can be reached on YouTube using this
            targeting.
    """

    locations: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=9,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    age_ranges: MutableSequence[criteria.AgeRangeInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=criteria.AgeRangeInfo,
    )
    gender: criteria.GenderInfo = proto.Field(
        proto.MESSAGE,
        number=3,
        message=criteria.GenderInfo,
    )
    parental_status: criteria.ParentalStatusInfo = proto.Field(
        proto.MESSAGE,
        number=8,
        message=criteria.ParentalStatusInfo,
    )
    user_interests: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadataGroup
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=11,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadataGroup,
    )
    coverage: float = proto.Field(
        proto.DOUBLE,
        number=5,
    )
    index: float = proto.Field(
        proto.DOUBLE,
        number=6,
    )
    potential_youtube_reach: int = proto.Field(
        proto.INT64,
        number=7,
    )


class ListAudienceInsightsAttributesRequest(proto.Message):
    r"""Request message for
    [AudienceInsightsService.ListAudienceInsightsAttributes][google.ads.googleads.v21.services.AudienceInsightsService.ListAudienceInsightsAttributes].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        dimensions (MutableSequence[google.ads.googleads.v21.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension]):
            Required. The types of attributes to be returned. Supported
            dimensions are CATEGORY, KNOWLEDGE_GRAPH, DEVICE,
            GEO_TARGET_COUNTRY, SUB_COUNTRY_LOCATION, YOUTUBE_LINEUP,
            AFFINITY_USER_INTEREST, IN_MARKET_USER_INTEREST,
            PARENTAL_STATUS, INCOME_RANGE, AGE_RANGE, and GENDER.
        query_text (str):
            Required. A free text query. If the requested dimensions
            include Attributes CATEGORY or KNOWLEDGE_GRAPH, then the
            attributes returned for those dimensions will match or be
            related to this string. For other dimensions, this field is
            ignored and all available attributes are returned.
        customer_insights_group (str):
            The name of the customer being planned for.
            This is a user-defined value.
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
        location_country_filters (MutableSequence[google.ads.googleads.v21.common.types.LocationInfo]):
            If SUB_COUNTRY_LOCATION attributes are one of the requested
            dimensions and this field is present, then the
            SUB_COUNTRY_LOCATION attributes returned will be located in
            these countries. If this field is absent, then location
            attributes are not filtered by country. Setting this field
            when SUB_COUNTRY_LOCATION attributes are not requested will
            return an error.
        youtube_reach_location (google.ads.googleads.v21.common.types.LocationInfo):
            If present, potential YouTube reach estimates within the
            specified market will be returned for attributes for which
            they are available. Reach is only available for the
            AGE_RANGE, GENDER, AFFINITY_USER_INTEREST and
            IN_MARKET_USER_INTEREST dimensions, and may not be available
            for every attribute of those dimensions in every market.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    dimensions: MutableSequence[
        audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
    ] = proto.RepeatedField(
        proto.ENUM,
        number=2,
        enum=audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension,
    )
    query_text: str = proto.Field(
        proto.STRING,
        number=3,
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=4,
    )
    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=7,
        message=additional_application_info.AdditionalApplicationInfo,
    )
    location_country_filters: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=5,
            message=criteria.LocationInfo,
        )
    )
    youtube_reach_location: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=6,
        message=criteria.LocationInfo,
    )


class ListAudienceInsightsAttributesResponse(proto.Message):
    r"""Response message for
    [AudienceInsightsService.ListAudienceInsightsAttributes][google.ads.googleads.v21.services.AudienceInsightsService.ListAudienceInsightsAttributes].

    Attributes:
        attributes (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata]):
            The attributes matching the search query.
    """

    attributes: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )


class ListInsightsEligibleDatesRequest(proto.Message):
    r"""Request message for
    [AudienceInsightsService.ListInsightsEligibleDates][google.ads.googleads.v21.services.AudienceInsightsService.ListInsightsEligibleDates].

    Attributes:
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=1,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class ListInsightsEligibleDatesResponse(proto.Message):
    r"""Response message for
    [AudienceInsightsService.ListInsightsEligibleDates][google.ads.googleads.v21.services.AudienceInsightsService.ListInsightsEligibleDates].

    Attributes:
        data_months (MutableSequence[str]):
            The months for which AudienceInsights data is
            currently available, each represented as a
            string in the form "YYYY-MM".
        last_thirty_days (google.ads.googleads.v21.common.types.DateRange):
            The actual dates covered by the "last 30 days" date range
            that will be used implicitly for
            [AudienceInsightsService.GenerateAudienceCompositionInsights][google.ads.googleads.v21.services.AudienceInsightsService.GenerateAudienceCompositionInsights]
            requests that have no data_month set.
    """

    data_months: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=1,
    )
    last_thirty_days: dates.DateRange = proto.Field(
        proto.MESSAGE,
        number=2,
        message=dates.DateRange,
    )


class GenerateAudienceOverlapInsightsRequest(proto.Message):
    r"""Request message for
    [AudienceInsightsService.GenerateAudienceOverlapInsights][google.ads.googleads.v21.services.AudienceInsightsService.GenerateAudienceOverlapInsights].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        country_location (google.ads.googleads.v21.common.types.LocationInfo):
            Required. The country in which to calculate
            the sizes and overlaps of audiences.
        primary_attribute (google.ads.googleads.v21.common.types.AudienceInsightsAttribute):
            Required. The audience attribute that should
            be intersected with all other eligible
            audiences.  This must be an Affinity or
            In-Market UserInterest, an AgeRange or a Gender.
        dimensions (MutableSequence[google.ads.googleads.v21.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension]):
            Required. The types of attributes of which to calculate the
            overlap with the primary_attribute. The values must be a
            subset of AFFINITY_USER_INTEREST, IN_MARKET_USER_INTEREST,
            AGE_RANGE and GENDER.
        customer_insights_group (str):
            The name of the customer being planned for.
            This is a user-defined value.
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    country_location: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=criteria.LocationInfo,
    )
    primary_attribute: audience_insights_attribute.AudienceInsightsAttribute = (
        proto.Field(
            proto.MESSAGE,
            number=6,
            message=audience_insights_attribute.AudienceInsightsAttribute,
        )
    )
    dimensions: MutableSequence[
        audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
    ] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum=audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension,
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=5,
    )
    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=7,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class GenerateAudienceOverlapInsightsResponse(proto.Message):
    r"""Response message for
    [AudienceInsightsService.GenerateAudienceOverlapInsights][google.ads.googleads.v21.services.AudienceInsightsService.GenerateAudienceOverlapInsights].

    Attributes:
        primary_attribute_metadata (google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata):
            Metadata for the primary attribute, including
            potential YouTube reach.
        dimension_results (MutableSequence[google.ads.googleads.v21.services.types.DimensionOverlapResult]):
            Lists of attributes and their overlap with
            the primary attribute, one list per requested
            dimension.
    """

    primary_attribute_metadata: (
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ) = proto.Field(
        proto.MESSAGE,
        number=3,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    dimension_results: MutableSequence["DimensionOverlapResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="DimensionOverlapResult",
        )
    )


class DimensionOverlapResult(proto.Message):
    r"""A list of audience attributes of a single dimension, including their
    overlap with a primary attribute, returned as part of a
    [GenerateAudienceOverlapInsightsResponse][google.ads.googleads.v21.services.GenerateAudienceOverlapInsightsResponse].

    Attributes:
        dimension (google.ads.googleads.v21.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension):
            The dimension of all the attributes in this
            section.
        items (MutableSequence[google.ads.googleads.v21.services.types.AudienceOverlapItem]):
            The attributes and their overlap with the
            primary attribute.
    """

    dimension: (
        audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension,
    )
    items: MutableSequence["AudienceOverlapItem"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="AudienceOverlapItem",
    )


class AudienceOverlapItem(proto.Message):
    r"""An audience attribute, with metadata including the overlap
    between this attribute's potential YouTube reach and that of a
    primary attribute.

    Attributes:
        attribute_metadata (google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata):
            The attribute and its metadata, including
            potential YouTube reach.
        potential_youtube_reach_intersection (int):
            The estimated size of the intersection of
            this audience attribute with the primary
            attribute, that is, the number of reachable
            YouTube users who match BOTH the primary
            attribute and this one.
    """

    attribute_metadata: (
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ) = proto.Field(
        proto.MESSAGE,
        number=3,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    potential_youtube_reach_intersection: int = proto.Field(
        proto.INT64,
        number=2,
    )


class GenerateTargetingSuggestionMetricsRequest(proto.Message):
    r"""Request message for
    [AudienceInsightsService.GenerateTargetingSuggestionMetrics][google.ads.googleads.v21.services.AudienceInsightsService.GenerateTargetingSuggestionMetrics].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        audiences (MutableSequence[google.ads.googleads.v21.services.types.InsightsAudience]):
            Required. Audiences to request metrics for.
        customer_insights_group (str):
            Optional. The name of the customer being
            planned for.  This is a user-defined value.
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    audiences: MutableSequence["InsightsAudience"] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message="InsightsAudience",
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=3,
    )
    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=4,
        message=additional_application_info.AdditionalApplicationInfo,
    )


class GenerateTargetingSuggestionMetricsResponse(proto.Message):
    r"""Response message for
    [AudienceInsightsService.GenerateTargetingSuggestionMetrics][google.ads.googleads.v21.services.AudienceInsightsService.GenerateTargetingSuggestionMetrics].

    Attributes:
        suggestions (MutableSequence[google.ads.googleads.v21.services.types.TargetingSuggestionMetrics]):
            Suggested targetable audiences. There will be one suggestion
            for each
            [GenerateTargetingSuggestionMetricsRequest.audiences]
            requested, matching the order requested.
    """

    suggestions: MutableSequence["TargetingSuggestionMetrics"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="TargetingSuggestionMetrics",
        )
    )


class BasicInsightsAudience(proto.Message):
    r"""A description of an audience used for requesting insights.

    Attributes:
        country_location (MutableSequence[google.ads.googleads.v21.common.types.LocationInfo]):
            Required. The countries for this audience.
        sub_country_locations (MutableSequence[google.ads.googleads.v21.common.types.LocationInfo]):
            Sub-country geographic location attributes.
            If present, each of these must be contained in
            one of the countries in this audience.
        gender (google.ads.googleads.v21.common.types.GenderInfo):
            Gender for the audience.  If absent, the
            audience does not restrict by gender.
        age_ranges (MutableSequence[google.ads.googleads.v21.common.types.AgeRangeInfo]):
            Age ranges for the audience.  If absent, the
            audience represents all people over 18 that
            match the other attributes.
        user_interests (MutableSequence[google.ads.googleads.v21.common.types.UserInterestInfo]):
            User interests defining this audience.
            Affinity and In-Market audiences are supported.
        topics (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsTopic]):
            Topics, represented by Knowledge Graph
            entities and/or Product & Service categories,
            that this audience is interested in.
    """

    country_location: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=criteria.LocationInfo,
        )
    )
    sub_country_locations: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=criteria.LocationInfo,
        )
    )
    gender: criteria.GenderInfo = proto.Field(
        proto.MESSAGE,
        number=3,
        message=criteria.GenderInfo,
    )
    age_ranges: MutableSequence[criteria.AgeRangeInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=criteria.AgeRangeInfo,
    )
    user_interests: MutableSequence[criteria.UserInterestInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=5,
            message=criteria.UserInterestInfo,
        )
    )
    topics: MutableSequence[
        audience_insights_attribute.AudienceInsightsTopic
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message=audience_insights_attribute.AudienceInsightsTopic,
    )


class InsightsAudienceDefinition(proto.Message):
    r"""A structured definition of the audience of interest for which
    insights are being requested in AudienceInsightsService.

    Attributes:
        audience (google.ads.googleads.v21.services.types.InsightsAudience):
            Required. The audience of interest for which
            insights are being requested.
        baseline_audience (google.ads.googleads.v21.services.types.InsightsAudience):
            Optional. The baseline audience. The default,
            if unspecified, is all people in the same
            country as the audience of interest.
        data_month (str):
            Optional. The one-month range of historical
            data to use for insights, in the format
            "yyyy-mm". If unset, insights will be returned
            for the last thirty days of data.
    """

    audience: "InsightsAudience" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="InsightsAudience",
    )
    baseline_audience: "InsightsAudience" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="InsightsAudience",
    )
    data_month: str = proto.Field(
        proto.STRING,
        number=3,
    )


class InsightsAudienceDescription(proto.Message):
    r"""A text description of the audience of interest for which
    insights are being requested in AudienceInsightsService.

    Attributes:
        country_locations (MutableSequence[google.ads.googleads.v21.common.types.LocationInfo]):
            Required. The countries for the audience.
        audience_description (str):
            Required. An English language text
            description of an audience to get suggestions
            for. Maximum length is 2000 characters. For
            example, "Women in their 30s who love to
            travel".
        marketing_objective (google.ads.googleads.v21.enums.types.AudienceInsightsMarketingObjectiveEnum.AudienceInsightsMarketingObjective):
            Optional. An optional marketing objective
            which will influence the type of suggestions
            produced. AWARENESS will provide affinity
            audience segments, while CONSIDERATION will
            provide in-market audience segments. Leaving it
            unset will provide both.
    """

    country_locations: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=criteria.LocationInfo,
        )
    )
    audience_description: str = proto.Field(
        proto.STRING,
        number=2,
    )
    marketing_objective: (
        audience_insights_marketing_objective.AudienceInsightsMarketingObjectiveEnum.AudienceInsightsMarketingObjective
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=audience_insights_marketing_objective.AudienceInsightsMarketingObjectiveEnum.AudienceInsightsMarketingObjective,
    )


class InsightsAudience(proto.Message):
    r"""A set of users, defined by various characteristics, for which
    insights can be requested in AudienceInsightsService.

    Attributes:
        country_locations (MutableSequence[google.ads.googleads.v21.common.types.LocationInfo]):
            Required. The countries for the audience.
        sub_country_locations (MutableSequence[google.ads.googleads.v21.common.types.LocationInfo]):
            Sub-country geographic location attributes. If present, each
            of these must be contained in one of the countries in this
            audience. If absent, the audience is geographically to the
            country_locations and no further.
        gender (google.ads.googleads.v21.common.types.GenderInfo):
            Gender for the audience.  If absent, the
            audience does not restrict by gender.
        age_ranges (MutableSequence[google.ads.googleads.v21.common.types.AgeRangeInfo]):
            Age ranges for the audience.  If absent, the
            audience represents all people over 18 that
            match the other attributes.
        parental_status (google.ads.googleads.v21.common.types.ParentalStatusInfo):
            Parental status for the audience.  If absent,
            the audience does not restrict by parental
            status.
        income_ranges (MutableSequence[google.ads.googleads.v21.common.types.IncomeRangeInfo]):
            Household income percentile ranges for the
            audience.  If absent, the audience does not
            restrict by household income range.
        lineups (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsLineup]):
            Lineups representing the YouTube content
            viewed by the audience.
        user_list (google.ads.googleads.v21.common.types.UserListInfo):
            User list to be targeted by the audience.
        topic_audience_combinations (MutableSequence[google.ads.googleads.v21.services.types.InsightsAudienceAttributeGroup]):
            A combination of entity, category and user
            interest attributes defining the audience. The
            combination has a logical AND-of-ORs structure:
            Attributes within each
            InsightsAudienceAttributeGroup are combined with
            OR, and the combinations themselves are combined
            together with AND.  For example, the expression
            (Entity OR Affinity) AND (In-Market OR Category)
            can be formed using two
            InsightsAudienceAttributeGroups with two
            Attributes each.
    """

    country_locations: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=criteria.LocationInfo,
        )
    )
    sub_country_locations: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=criteria.LocationInfo,
        )
    )
    gender: criteria.GenderInfo = proto.Field(
        proto.MESSAGE,
        number=3,
        message=criteria.GenderInfo,
    )
    age_ranges: MutableSequence[criteria.AgeRangeInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=criteria.AgeRangeInfo,
    )
    parental_status: criteria.ParentalStatusInfo = proto.Field(
        proto.MESSAGE,
        number=5,
        message=criteria.ParentalStatusInfo,
    )
    income_ranges: MutableSequence[criteria.IncomeRangeInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message=criteria.IncomeRangeInfo,
        )
    )
    lineups: MutableSequence[
        audience_insights_attribute.AudienceInsightsLineup
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message=audience_insights_attribute.AudienceInsightsLineup,
    )
    user_list: criteria.UserListInfo = proto.Field(
        proto.MESSAGE,
        number=11,
        message=criteria.UserListInfo,
    )
    topic_audience_combinations: MutableSequence[
        "InsightsAudienceAttributeGroup"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message="InsightsAudienceAttributeGroup",
    )


class InsightsAudienceAttributeGroup(proto.Message):
    r"""A list of AudienceInsightsAttributes.

    Attributes:
        attributes (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttribute]):
            Required. A collection of audience attributes
            to be combined with logical OR. Attributes need
            not all be the same dimension.  Only Knowledge
            Graph entities, Product & Service Categories,
            and Affinity and In-Market audiences are
            supported in this context.
    """

    attributes: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttribute
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=audience_insights_attribute.AudienceInsightsAttribute,
    )


class AudienceCompositionSection(proto.Message):
    r"""A collection of related attributes of the same type in an
    audience composition insights report.

    Attributes:
        dimension (google.ads.googleads.v21.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension):
            The type of the attributes in this section.
        top_attributes (MutableSequence[google.ads.googleads.v21.services.types.AudienceCompositionAttribute]):
            The most relevant segments for this audience. If dimension
            is GENDER, AGE_RANGE or PARENTAL_STATUS, then this list of
            attributes is exhaustive.
        clustered_attributes (MutableSequence[google.ads.googleads.v21.services.types.AudienceCompositionAttributeCluster]):
            Additional attributes for this audience, grouped into
            clusters. Only populated if dimension is YOUTUBE_CHANNEL.
    """

    dimension: (
        audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension,
    )
    top_attributes: MutableSequence["AudienceCompositionAttribute"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message="AudienceCompositionAttribute",
        )
    )
    clustered_attributes: MutableSequence[
        "AudienceCompositionAttributeCluster"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="AudienceCompositionAttributeCluster",
    )


class AudienceCompositionAttributeCluster(proto.Message):
    r"""A collection of related attributes, with metadata and
    metrics, in an audience composition insights report.

    Attributes:
        cluster_display_name (str):
            The name of this cluster of attributes
        cluster_metrics (google.ads.googleads.v21.services.types.AudienceCompositionMetrics):
            If the dimension associated with this cluster is
            YOUTUBE_CHANNEL, then cluster_metrics are metrics associated
            with the cluster as a whole. For other dimensions, this
            field is unset.
        attributes (MutableSequence[google.ads.googleads.v21.services.types.AudienceCompositionAttribute]):
            The individual attributes that make up this
            cluster, with metadata and metrics.
    """

    cluster_display_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster_metrics: "AudienceCompositionMetrics" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="AudienceCompositionMetrics",
    )
    attributes: MutableSequence["AudienceCompositionAttribute"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=4,
            message="AudienceCompositionAttribute",
        )
    )


class AudienceCompositionMetrics(proto.Message):
    r"""The share and index metrics associated with an attribute in
    an audience composition insights report.

    Attributes:
        baseline_audience_share (float):
            The fraction (from 0 to 1 inclusive) of the
            baseline audience that match the attribute.
        audience_share (float):
            The fraction (from 0 to 1 inclusive) of the
            specific audience that match the attribute.
        index (float):
            The ratio of audience_share to baseline_audience_share, or
            zero if this ratio is undefined or is not meaningful.
        score (float):
            A relevance score from 0 to 1 inclusive.
    """

    baseline_audience_share: float = proto.Field(
        proto.DOUBLE,
        number=1,
    )
    audience_share: float = proto.Field(
        proto.DOUBLE,
        number=2,
    )
    index: float = proto.Field(
        proto.DOUBLE,
        number=3,
    )
    score: float = proto.Field(
        proto.DOUBLE,
        number=4,
    )


class AudienceCompositionAttribute(proto.Message):
    r"""An audience attribute with metadata and metrics.

    Attributes:
        attribute_metadata (google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata):
            The attribute with its metadata.
        metrics (google.ads.googleads.v21.services.types.AudienceCompositionMetrics):
            Share and index metrics for the attribute.
    """

    attribute_metadata: (
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ) = proto.Field(
        proto.MESSAGE,
        number=3,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    metrics: "AudienceCompositionMetrics" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="AudienceCompositionMetrics",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import dates
from google.ads.googleads.v22.enums.types import device as gage_device
from google.ads.googleads.v22.enums.types import (
    keyword_plan_aggregate_metric_type,
)
from google.ads.googleads.v22.enums.types import keyword_plan_competition_level
from google.ads.googleads.v22.enums.types import keyword_plan_concept_group_type
from google.ads.googleads.v22.enums.types import month_of_year


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.common",
    marshal="google.ads.googleads.v22",
    manifest={
        "KeywordPlanHistoricalMetrics",
        "HistoricalMetricsOptions",
        "MonthlySearchVolume",
        "KeywordPlanAggregateMetrics",
        "KeywordPlanAggregateMetricResults",
        "KeywordPlanDeviceSearches",
        "KeywordAnnotations",
        "KeywordConcept",
        "ConceptGroup",
    },
)


class KeywordPlanHistoricalMetrics(proto.Message):
    r"""Historical metrics specific to the targeting options
    selected. Targeting options include geographies, network, and so
    on. Refer to
    https://support.google.com/google-ads/answer/3022575 for more
    details.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        avg_monthly_searches (int):
            Approximate number of monthly searches on
            this query, averaged for the past 12 months.

            This field is a member of `oneof`_ ``_avg_monthly_searches``.
        monthly_search_volumes (MutableSequence[google.ads.googleads.v22.common.types.MonthlySearchVolume]):
            Approximate number of searches on this query
            for the past twelve months.
        competition (google.ads.googleads.v22.enums.types.KeywordPlanCompetitionLevelEnum.KeywordPlanCompetitionLevel):
            The competition level for the query.
        competition_index (int):
            The competition index for the query in the range [0, 100].
            Shows how competitive ad placement is for a keyword. The
            level of competition from 0-100 is determined by the number
            of ad slots filled divided by the total number of ad slots
            available. If not enough data is available, null is
            returned.

            This field is a member of `oneof`_ ``_competition_index``.
        low_top_of_page_bid_micros (int):
            Top of page bid low range (20th percentile)
            in micros for the keyword.

            This field is a member of `oneof`_ ``_low_top_of_page_bid_micros``.
        high_top_of_page_bid_micros (int):
            Top of page bid high range (80th percentile)
            in micros for the keyword.

            This field is a member of `oneof`_ ``_high_top_of_page_bid_micros``.
        average_cpc_micros (int):
            Average Cost Per Click in micros for the
            keyword.

            This field is a member of `oneof`_ ``_average_cpc_micros``.
    """

    avg_monthly_searches: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    monthly_search_volumes: MutableSequence["MonthlySearchVolume"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message="MonthlySearchVolume",
        )
    )
    competition: (
        keyword_plan_competition_level.KeywordPlanCompetitionLevelEnum.KeywordPlanCompetitionLevel
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=keyword_plan_competition_level.KeywordPlanCompetitionLevelEnum.KeywordPlanCompetitionLevel,
    )
    competition_index: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )
    low_top_of_page_bid_micros: int = proto.Field(
        proto.INT64,
        number=9,
        optional=True,
    )
    high_top_of_page_bid_micros: int = proto.Field(
        proto.INT64,
        number=10,
        optional=True,
    )
    average_cpc_micros: int = proto.Field(
        proto.INT64,
        number=11,
        optional=True,
    )


class HistoricalMetricsOptions(proto.Message):
    r"""Historical metrics options.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        year_month_range (google.ads.googleads.v22.common.types.YearMonthRange):
            The year month range for historical metrics. If not
            specified, metrics for the past 12 months are returned.
            Search metrics are available for the past 4 years. If the
            search volume is not available for the entire
            year_month_range provided, the subset of the year month
            range for which search volume is available are returned.

            This field is a member of `oneof`_ ``_year_month_range``.
        include_average_cpc (bool):
            Indicates whether to include average cost per
            click value. Average CPC is provided only for
            legacy support.
    """

    year_month_range: dates.YearMonthRange = proto.Field(
        proto.MESSAGE,
        number=1,
        optional=True,
        message=dates.YearMonthRange,
    )
    include_average_cpc: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class MonthlySearchVolume(proto.Message):
    r"""Monthly search volume.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        year (int):
            The year of the search volume (for example,
            2020).

            This field is a member of `oneof`_ ``_year``.
        month (google.ads.googleads.v22.enums.types.MonthOfYearEnum.MonthOfYear):
            The month of the search volume.
        monthly_searches (int):
            Approximate number of searches for the month.
            A null value indicates the search volume is
            unavailable for that month.

            This field is a member of `oneof`_ ``_monthly_searches``.
    """

    year: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    month: month_of_year.MonthOfYearEnum.MonthOfYear = proto.Field(
        proto.ENUM,
        number=2,
        enum=month_of_year.MonthOfYearEnum.MonthOfYear,
    )
    monthly_searches: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )


class KeywordPlanAggregateMetrics(proto.Message):
    r"""The aggregate metrics specification of the request.

    Attributes:
        aggregate_metric_types (MutableSequence[google.ads.googleads.v22.enums.types.KeywordPlanAggregateMetricTypeEnum.KeywordPlanAggregateMetricType]):
            The list of aggregate metrics to fetch data.
    """

    aggregate_metric_types: MutableSequence[
        keyword_plan_aggregate_metric_type.KeywordPlanAggregateMetricTypeEnum.KeywordPlanAggregateMetricType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=1,
        enum=keyword_plan_aggregate_metric_type.KeywordPlanAggregateMetricTypeEnum.KeywordPlanAggregateMetricType,
    )


class KeywordPlanAggregateMetricResults(proto.Message):
    r"""The aggregated historical metrics for keyword plan keywords.

    Attributes:
        device_searches (MutableSequence[google.ads.googleads.v22.common.types.KeywordPlanDeviceSearches]):
            The aggregate searches for all the keywords
            segmented by device for the specified time.
            Supports the following device types: MOBILE,
            TABLET, DESKTOP.

            This is only set when
            KeywordPlanAggregateMetricTypeEnum.DEVICE is set
            in the KeywordPlanAggregateMetrics field in the
            request.
    """

    device_searches: MutableSequence["KeywordPlanDeviceSearches"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="KeywordPlanDeviceSearches",
        )
    )


class KeywordPlanDeviceSearches(proto.Message):
    r"""The total searches for the device type during the specified
    time period.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        device (google.ads.googleads.v22.enums.types.DeviceEnum.Device):
            The device type.
        search_count (int):
            The total searches for the device.

            This field is a member of `oneof`_ ``_search_count``.
    """

    device: gage_device.DeviceEnum.Device = proto.Field(
        proto.ENUM,
        number=1,
        enum=gage_device.DeviceEnum.Device,
    )
    search_count: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )


class KeywordAnnotations(proto.Message):
    r"""The annotations for the keyword plan keywords.

    Attributes:
        concepts (MutableSequence[google.ads.googleads.v22.common.types.KeywordConcept]):
            The list of concepts for the keyword.
    """

    concepts: MutableSequence["KeywordConcept"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="KeywordConcept",
    )


class KeywordConcept(proto.Message):
    r"""The concept for the keyword.

    Attributes:
        name (str):
            The concept name for the keyword in the concept_group.
        concept_group (google.ads.googleads.v22.common.types.ConceptGroup):
            The concept group of the concept details.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    concept_group: "ConceptGroup" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ConceptGroup",
    )


class ConceptGroup(proto.Message):
    r"""The concept group for the keyword concept.

    Attributes:
        name (str):
            The concept group name.
        type_ (google.ads.googleads.v22.enums.types.KeywordPlanConceptGroupTypeEnum.KeywordPlanConceptGroupType):
            The concept group type.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    type_: (
        keyword_plan_concept_group_type.KeywordPlanConceptGroupTypeEnum.KeywordPlanConceptGroupType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=keyword_plan_concept_group_type.KeywordPlanConceptGroupTypeEnum.KeywordPlanConceptGroupType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
